c     ------------------------------------------------------------------
c     Authors:
c     E. J. Valeo and  D. P. Enright, August 1992
c     Modified for loading with LSC by D. W. Ignat, October 1992
c     Modified August 24 1993 by Ignat to get w/cm^2; also fix some
c     indexing errors.
c     Copyright:
c     Princeton University, Plasma Physics Laboratory, 1992, 1993, by
c     E. J. Valeo, D. W. Ignat, S. von Goeler, S. C. Jardin, 
c     P. G. Roney, J. E. Stevens, and D. P. Enright
c
c     Units: Energy in MeV mostly; kev in absorber routines;
c     mc^2 in the evaluation of Koch and Motz formula.
c     Bremsstrahlung photon radiation in w/cm^2 on camera; w/cm^3 in plasma
c
c     References:
c     H. W. Koch and J. W. Motz, 
c     ``Bremsstrahlung Cross-Section Formulas and Related Data,''
c     \RMP{31}{920}{59}.
c     Formula 2BN on p 924
c     '2' denotes differential in photon energy and angle
c     'B' denotes Born approximation
c     'N' denotes no screening
c     and
c     X ray crosss sections based on W. H. McMaster, el al, UCRL 50174
c     Sec. II rev. 1,  May 1969.
c
c     .         --------------------------------------------------------
c
#ifndef USE_X_RAY_CAMERA
      SUBROUTINE XrayCam2
      return 
      END
      SUBROUTINE XRopen(i,a)
      INTEGER i
      CHARACTER*24 a
      return 
      ENTRY XRclos(i)
      return 
      END
      SUBROUTINE XRdscw(x, y, n, a)
#include "Implic.inc"
      INTEGER n
      REAL    x(n), y(n)
      CHARACTER*(*) a
      return 
      END
c      SUBROUTINE XRdscr
c      return 
c      END
#else
#define float dble
      SUBROUTINE XrayCam2
#include "Implic.inc"
#include "params.inc"
#include "xparams.inc"
#include "RayBins.inc"
#include "FeBins.inc"
#include "ProfBody.inc"
      call xRayInp
      call xBremSetup(ZbrAry(1))
      call xRaySetup
      call xIntens(NVELDIM,NPSIDIM,iITR,fe, nv, 
     ^             npsi, NeAry, ZbrAry, Vpar)
      call xEmit
c
      return
      END
c
c     ------------------------------------------------------------------
      SUBROUTINE xRayInp
#include "Implic.inc"
#include "params.inc"
#include "TSCgrap.inc"
#include "tscunits.inc"
#include "emparams.inc"
#include "xparams.inc"
#include "Xray.inc"
#include "emitter.inc"
#include "camera.inc"
#include "numerics.inc"
C     NameList begins                       ---------------------------|
C                                                                      |
C                                                                      |
      NAMELIST /inpxry/ 
     ^        dFoilTCM, FoilCode, iAbsXray,
     ^        nEbins, nMUbins, E_max, E_min, nr_source, nz_source,
     ^        Z_bound_min, Z_bound_max, R_bound_max, R_bound_min,
     ^        n_pixel_x, n_pixel_y,
     ^        E_ph_min, dE_ph,
     ^        Rpinhole, Zpinhole, phi_pinhole, pinhole_size,
     ^        focal_length, screen_d, R_tangent, z_tangent,
     ^        dlxray, npoints_max,
     ^        PusherMajor, PusherMinor 
C                                                                      |
C                                                                      |
C     NameList ends                         ---------------------------|
c
c#include "emitter.inc"
c
      Z_plasm_max = amax1(ZlcfsMax,-ZlcfsMin)
      Z_plasm_min = -Z_plasm_max
      R_plasm_max = RlcfsMax
      R_plasm_min = RlcfsMin

      Z_bound_max = Z_plasm_max*1.005
      Z_bound_min =-Z_plasm_max*1.005
      R_bound_max = RlcfsMax*1.005
      R_bound_min = amin1(RlcfsMin/1.005,R_bound_min)
c
c#include "camera.inc"
c
c
c#include "numerics.inc"
      open (nTSCunus,status='old',file = 'input.xry', err=1300 )
      read (nTSCunus, inpxry)
      close (nTSCunus)
c
      goto 1301
 1300 call LSCwarn (' cant find input.xry; use defaults! ')
 1301 continue
      if (n_pixel_x .gt. NPIXDIM) then
        call LSCwarn('n_pixel_x reset to NPIXDIM')
        n_pixel_x = NPIXDIM
        endif

      if (n_pixel_y .gt. NPIXDIM) then
        call LSCwarn('n_pixel_y reset to NPIXDIM')
        n_pixel_y = NPIXDIM
        endif

      if (nr_source .gt. NRDIM) then
        call LSCwarn('nr_source reset to NRDIM')
        nr_source = NRDIM
        endif

      if (nz_source .gt. NZDIM) then
        call LSCwarn('nz_source reset to NZDIM')
        nz_source = NZDIM
        endif

      if (nMUbins .gt. NMUDIM) then
        call LSCwarn('nMUbins reset to NMUDIM')
        nMUbins = NMUDIM
        endif
        
      if (2*(nMUbins/2) .eq. nMUbins) then
        call LSCwarn('nMUbins must be odd; subtracting 1')        
        nMUbins = nMUbins-1
        endif

      if (nEbins .gt. NENDIM) then
        call LSCwarn('nEbins reset to NENDIM')
        nEbins = NENDIM
        endif

      if (E_max .gt. 0.255) then
        call LSCwarn('E_max reset to 0.255 MeV')
        E_max = 0.255 
        endif
c
      pix_size_x = screen_d/n_pixel_x
      pix_size_y = screen_d/n_pixel_y
c
      call GrafFixP('BEGIN PLOT', 0    )
      call GrafFixP(' NRDIM    ', NRDIM)
      call GrafFixP(' NZDIM    ', NZDIM)
      call GrafFixP(' NRZMXDIM ', NRZMXDIM)
      call GrafFixP(' NPIXDIM  ', NPIXDIM)
      call GrafFixP(' NRCHORDIM', NCHORDIM)
      call GrafFixP(' MAXPOINTS', MAXPOINTS)
      call GrafFixP(' SPACEDIM ', SPACEDIM)
      call GrafFltP(' ACC_ERROR', ACCEPTABLE_ERROR)
      call GrafFixP(' NMUDIM   ', NMUDIM)
      call GrafFixP(' NENDIM   ', NENDIM)
      call GrafFixP(' iAbsXray ', iAbsXray)
      call GrafChrS(' FoilCode ', FoilCode)
      call GrafFltP(' dFoilTCM ', dFoilTCM)
      call GrafFixP(' nMUbins  ', nMUbins)
      call GrafFixP(' nEbins   ', nEbins)
      call GrafFltP(' mu_min   ', mu_min)
      call GrafFltP(' mu_max   ', mu_max)
      call GrafFltP(' E_min    ', E_min)
      call GrafFltP(' E_max    ', E_max)
      call GrafFltP(' E_ph_min ', E_ph_min)
      call GrafFltP(' dE_ph    ', dE_ph)
      call GrafFixP(' nr_source', nr_source)
      call GrafFixP(' nz_source', nz_source)
      call GrafFltP(' mu_0     ', mu_0)
      call GrafFltP(' mu_width ', mu_width)
      call GrafFltP(' ZboundMin', Z_bound_min)
      call GrafFltP(' ZboundMax', Z_bound_max)
      call GrafFltP(' RboundMin', R_bound_min)
      call GrafFltP(' RboundMax', R_bound_max)
      call GrafFltP(' ZplasmMin', Z_plasm_min)
      call GrafFltP(' ZplasmMax', Z_plasm_max)
      call GrafFltP(' RplasmMin', R_plasm_min)
      call GrafFltP(' RplasmMax', R_plasm_max)
      call GrafFltP(' ZlcfsMin ', ZlcfsMin)
      call GrafFltP(' ZlcfsMax ', ZlcfsMax)
      call GrafFltP(' RlcfsMin ', RlcfsMin)
      call GrafFltP(' RlcfsMax ', RlcfsMax)
      call GrafFltP(' PusherMaj', PusherMajor)
      call GrafFltP(' PusherMin', PusherMinor)
      call GrafFixP(' n_pixel_x', n_pixel_x)
      call GrafFixP(' n_pixel_y', n_pixel_y)
      call GrafFltP(' focal_len', focal_length)
      call GrafFltP(' screen_d ', screen_d)
      call GrafFltP(' pinhole_s', pinhole_size)
      call GrafFltP(' R_tangent', R_tangent)
      call GrafFltP(' Z_tangent', Z_tangent)
      call GrafFltP(' RcntChrd ', RcntChrd)
      call GrafFltP(' Rpinhole ', Rpinhole)
      call GrafFltP(' Zpinhole ', Zpinhole)
      call GrafFltP(' pix_sizex', pix_size_x)
      call GrafFltP(' pix_sizey', pix_size_y)
      call GrafFixP(' npointsMx', npoints_max)
      call GrafFltP(' dlxray   ', dlxray)
c     call GrafFixP(' .', .)
c     call GrafFltP(' .', .)
c     call GrafChrS(' .', .)
      call MkGrfLst(' List X ray Camera parameters ')
      call GrafFixP('END PLOT  ', 0   )
c
      return
      END
c     ..................................................................
      BLOCK DATA xRayBk
#include "Implic.inc"
#include "params.inc"
#include "xparams.inc"
#include "emparams.inc"
#include "Xray.inc"

#include "emitter.inc"

#include "camera.inc"

#include "numerics.inc"

      DATA mu_min, mu_max / -1., 1. / 
      DATA nMUbins / NMUDIM /
      DATA nEbins  / NENDIM / 
      DATA E_ph_min / 1.e-6 / 
      DATA dE_ph / .01 / 
      DATA E_min, E_max / 0.01 , 0.2 /
      DATA Z_bound_min, Z_bound_max /
     ^           -1.00,        1.00 / 
      DATA R_bound_min, R_bound_max / 
     ^            1.10,        2.00 /
      DATA mu_0 / 1. / mu_width / .2 /
      DATA nr_source / NRDIM / nz_source / NZDIM /

      DATA Rpinhole / 2.661 / 
      DATA Zpinhole /   0. /
      DATA phi_pinhole / 0. / 
      DATA pinhole_size / 0.00625 /
      DATA focal_length, screen_d  / 
     ^          0.4905,  0.215     /
      DATA n_pixel_x, n_pixel_y  / NPIXDIM, NPIXDIM / 
      DATA R_tangent / 1.524 / 
      DATA RcntChrd  / 1.65  /
      DATA z_tangent / 0. /
      DATA pix_size_x, pix_size_y / 0.01, 0.01 /

      DATA npoints_max / MAXPOINTS / 
      DATA eps / ACCEPTABLE_ERROR / 
      DATA dlxray / 0.005 /
      DATA  PusherMajor, PusherMinor / 1.152, 0.190 /
      DATA FoilCode / 'AG' /
      DATA dFoilTCM / 0.000 /
      DATA iAbsXray / 1 /
c
      END
c     .         --------------------------------------------------------
      SUBROUTINE xRaySetup 
#include "Implic.inc"
#include "params.inc"
#include "FeBins.inc"
#include "xparams.inc"
#include "Xray.inc"
#include "emparams.inc"
#include "emitter.inc"
c     generate grids, cross-section database
c     and adjust the sigtot by the transmission and illumination factor
c
      call ugrid(muvec, nMUbins, mu_min, mu_max)
      dmu_inv = float(nMUbins - 1) / (mu_max - mu_min)
c
      call ugrid(E_incvec, nEbins, E_min, E_max)
      call xSetupEindex
c     set up indices for table lookup into sigma
      call xGenEofv(nv, Eofv, dEofv, Vpar)
      if (iAbsXray .gt. 0) then
        call GetXmnFac(nEbins, E_incvec, XmnFac, dFoilTCM, FoilCode)
c        call lscpause
c        call vwrite(nEbins, E_incvec, 'E_incvec')
c        call vwrite(nEbins, XmnFac, 'XmnFac')
c        call lscpause
      endif
c     compute vector XmnFac of absorption coefficients
      call xSigma

      return
      END
c     .         --------------------------------------------------------
      SUBROUTINE xGenEofv(n, E, dE, v)
#include "Implic.inc"
      INTEGER n, i
      REAL   signorm, Mevpermcsq
      COMMON / sigbrcom0 / signorm, Mevpermcsq
      REAL   E(n), dE(n), v(n)
      do i = 1, n
         E(i) = 0.5 * Mevpermcsq * v(i) * v(i)
      enddo
      do i = 2, n - 1
         dE(i) = 0.5 * abs(E(i + 1) - E(i - 1))
c     accounts for negative velocity
      enddo
      dE(1) = 0.5 * abs(E(2) - E(1))
      dE(n) = 0.5 * abs(E(n) - E(n - 1))
      return
      END
c     .         --------------------------------------------------------
      SUBROUTINE xSetupEindex
#include "Implic.inc"
#include "params.inc"
#include "xparams.inc"
#include "FeBins.inc"      
#include "Xray.inc"
      INTEGER i, j
      REAL   Einc, MevPerMcsq
      PARAMETER ( MevPerMcsq = 0.511) 

      do i = 1, nv
         Einc = 0.5 * Mevpermcsq * Vpar(i) * Vpar(i)
         j = 1  
         do while((E_incvec(j) .le. Einc) .and. (j .lt. nEbins))
            j = j + 1
         enddo
         j = j - 1
         Eint(i) = j
         if(j .ge. 1 .and. j .lt. nEbins) then
           Efrac(i) = (Einc - E_incvec(j)) / (E_incvec(j + 1) -
     ^        E_incvec(j))
         else
           Efrac(i) = 0.00
         endif
      enddo
      return
      END
c     .         --------------------------------------------------------
      REAL FUNCTION Xraytransmission(EmeV)
#include "Implic.inc"
#include "params.inc"
#include "xparams.inc"
#include "Xray.inc"
      INTEGER ie
      REAL frac, EmeV
      if(iAbsXray .gt. 0)then
         call Lookupindex(EmeV, E_incvec, nEbins, ie, frac)
         Xraytransmission = XmnFac(ie) * (1. - frac) +
     $        XmnFac(ie + 1) * frac
      else
         Xraytransmission = 1.
      endif
      return
      END
c     .         --------------------------------------------------------
      SUBROUTINE Lookupindex(c, vec, n, i, frac)
#include "Implic.inc"
      INTEGER n, i
      REAL 
     ^        c, vec(n), frac, ri
      real    RE41
      ri = (c - vec(1)) / (vec(2) - vec(1))
c     i = ifix(ri)
      RE41 = ri    
      i = ifix(RE41)
      frac = ri - float(i)
      if(i .lt. 1)then
         i = 1
         frac = 0.
      else if(i .ge. n)then
         i = n - 1
         frac = 1.
      endif
      return
      END
c     .         --------------------------------------------------------
      SUBROUTINE xSigma
#include "Implic.inc"
#include "params.inc"
#include "xparams.inc"
#include "Xray.inc"
      REAL    sigg, E_photon, Xraytransmission
      INTEGER  jj, kk
      do 10 kk = 1, nEbins

         do 20 jj = 1, nMUbins
            sigtot(jj,kk) = 0.
            E_photon = E_ph_min + 0.5 * dE_ph 
            do while(E_photon .lt. E_incvec(kk))
               call xSigGen(E_incvec(kk), E_photon, muvec(jj),
     ^              sigg)
               sigtot(jj,kk) = sigtot(jj,kk) + sigg * E_photon * dE_ph
     $              * Xraytransmission(E_photon       )
c     weighted to yield rate of energy emission.
c
               E_photon = E_photon + dE_ph
            enddo
 20      continue
 10   continue   

cc    Ernies contour plot of sigtot contours 28Mar93 begin
c     call X_pl_intensity_cntrs(sigtot, NMUDIM, nMUbins, NEbins)
c     call EZfini(0,0)
cc    Ernies contour plot of sigtot contours 28Mar93 begin

      return
      END
      
c     +---------------------------------------------------------------+
      
      SUBROUTINE xIntens(NVDIM,NPDIM,iIterate,FeOfv, nv, np,
     $     NeAry, ZbrAry, Vpar)
c     intensity subroutine calculates the value of the photon intensity
c     over a range of scattering angles mu and for each psi flux surface
c     Actually, the values given are I(mu,psi)/Clight.  
      
#include "Implic.inc"
#include "params.inc"
#include "xparams.inc"
#include "Xray.inc"
      INTEGER NPDIM, np, NVDIM, nv, iv, j, ip, iIterate
      INTEGER iv0
      INTEGER ith
      REAL    FeOfv(NVDIM, NPDIM, 2), sig_th(NMUDIM), Ifactor
      REAL    NeAry(NPDIM), ZbrAry(NPDIM), Vpar(NVDIM)
c     PARAMETER (MevPerMcsq = 0.511) delete this; add lines below
      REAL   signorm, Mevpermcsq
      COMMON / sigbrcom0 / signorm, Mevpermcsq
      REAL    
     ^       ONE
      DATA   ONE/
     ^       1.0/
      iv0 = (nv + 1) / 2
c     velocity index for which Vpar(iv0) = 0
c     mu is measured relative to positive Vpar axis.
c     sigtot   INPUT  differential cross section times photon energy
c                     times transmission factor (.le. 1.0)
c                     w.r.t. solid angle(str) in
c                     MeV - cm^2 per atom per incident electron
c     sig_th   OUTPUT differential cross section times photon energy
c                     times transmission factor (.le. 1.0)
c                     w.r.t. solid angle(str) interpolated velocity in
c                     MeV - cm^2 per atom per incident electron
c     inten    OUTPUT joules/sec per cm^3 per steradian

      call BrodCast(NMUDIM * NPDIM, inten, 0.)
      do 100 iv = 1, nv
c.
        do 10 ith = 1, nMUbins
          if(Eint(iv) .ge. 1 .and. Eint(iv) .lt. nEbins) then
            sig_th(ith) = sigtot(ith, Eint(iv)) * (1. - Efrac(iv)) +
     ^                    Efrac(iv) *sigtot(ith, Eint(iv) + 1)
          else
            sig_th(ith) = 0.00
          endif
 10     continue
c.
        if(iv .le. iv0) then
          call xVecRefl(nMUbins, sig_th)
          Ifactor = 0.5 * abs((Vpar(iv + 1) + Vpar(iv)) *
     $         (Vpar(iv + 1) - Vpar(iv)))
       else
          Ifactor = 0.5 * abs((Vpar(iv) + Vpar(iv - 1)) *
     $         (Vpar(iv) - Vpar(iv - 1)))
        endif
c     note: dEofv ALWAYS .gt. 0, and Vpar(iv) increases from (-1., 1.),
c     
        do 50 ip = 1, np
          do 40 j = 1, nMUbins
            inten(j, ip) = inten(j, ip) + 
     ^                     FeOfv(iv, ip, iIterate) * sig_th(j) *
     ^                     Ifactor
 40      continue 
 50     continue
 100  continue
c
      do 150 ip = 1, np
        do 140 j = 1, nMUbins
          inten(j, ip) = inten(j, ip) * NeAry(ip) 
     ^                   / amax1(ONE ,ZbrAry(ip)) * 4.8e-3
c    ^                   / amax1(1.00,ZbrAry(ip)) * 4.8e-3

 140    continue
 150  continue
      return
      END
      
c     +---------------------------------------------------------------+
      
      SUBROUTINE xVecRefl(n, vec)
#include "Implic.inc"
      INTEGER i, n, i_comp
      REAL    vec(n), vec_tmp
      do i = 1, (n + 1) / 2
         i_comp = n + 1 - i
         vec_tmp = vec(i_comp)
         vec(i_comp) = vec(i)
         vec(i) = vec_tmp
      enddo
      return
      END

c     +---------------------------------------------------------------+
      
      REAL   FUNCTION delta_em(rr, zz, mu)
#include "Implic.inc"
#include "params.inc"
#include "RayWrk.inc"
      INTEGER ip, imu
      REAL    rr, zz, mu, ip_frac, imu_frac
      call xGetPsi(rr, zz, mu, ip, ip_frac, imu, imu_frac)
      call xPsiIntp(ip, ip_frac, imu, imu_frac, delta_em)
      delta_em = delta_em * Rmaj / rr
c     to account for convergence of flux tubes
      return
      END
c     .         --------------------------------------------------------
      SUBROUTINE xGetPsi(r, z, mu, ip, ip_frac, imu, imu_frac)
c     find psi of r,z
#include "Implic.inc"
#include "params.inc"
#include "RayBins.inc"
#include "ProfBody.inc"
#include "TSCgrap.inc"
#include "emparams.inc"
#include "xparams.inc"
#include "Xray.inc"
      INTEGER ip, imu, jlo, jhi
      REAL    mu, ip_frac, imu_frac, rmu
      REAL    r,z,psi,Br,Bz,RBphi,omc,Tee,pe2,pi2,aio,ael
      DATA    jlo / -1 /
      call plasma2d(r,z,psi,Br,Bz,RBphi,omc,Tee,pe2,pi2,aio,ael)
      if (psi .ge. PsiAry(npsi) .or. pe2 .le. 0.00 .or.
     ^      r .ge. RlcfsMax     .or.   r .le. RlcfsMin .or.
     ^      z .le. ZlcfsMin     .or.   z .ge. ZlcfsMax ) then
        ip      = 0
        ip_frac = 0.00
        imu     = 1
        imu_frac= 0.00
        return
      endif
      call huntnr(PsiAry, npsi, psi, jlo)
      ip = jlo
      jhi= jlo+1
      if (jhi .gt. npsi) then
        ip_frac = 0.00
      else
        ip_frac = (psi - PsiAry(jlo))/(PsiAry(jlo+1) - PsiAry(jlo))
      endif
      rmu = (mu - mu_min) * dmu_inv + 1.00
      imu = int(rmu)
      imu_frac = rmu - float(imu)
      return
      END

      SUBROUTINE xPsiIntp(ip, ip_frac, imu, imu_frac, delta_em)
#include "Implic.inc"
#include "params.inc"
#include "RayBins.inc"
#include "xparams.inc"
#include "Xray.inc"
      INTEGER ip, imu, ip_plus, imu_plus
      REAL    ip_frac, imu_frac, delta_em
c     linear intepolation
      imu_plus = min(imu + 1, nMUbins)
      ip_plus = min(ip + 1, npsi)
      if (ip .le. 0 ) then
        delta_em = 0.00
      else
        delta_em = (1. - ip_frac - imu_frac) * inten(imu, ip)
     ^     + ip_frac * inten(imu, ip_plus) 
     ^     + imu_frac *inten(imu_plus, ip)
      endif
      return
      END
c
c     emit.F          --------------------------------------------------
c
c     produce bremstrahlung emission pattern from prescribed source
c     as observed by X-ray pinhole camera

      SUBROUTINE xEmit
#include "Implic.inc"
cNCAR      call opngks
           call EZinit        
      call X_setup_camera
      call X_compute_pattern
      call X_diagnose_trajectories
      call X_plot_pixel_data
      call X_plot_source_profile
           call MkGrfLst('X ray contour plots')
           call EZfini(0,0)
c!SLICES HERE?
           call EZinit        
      call xSlices
      call eSlices
c     call XmnGraf

c     call X_wrres
cNCAR      call clsgks
           call EZfini(0,0)

      return
      END
c     .         --------------------------------------------------------
      SUBROUTINE X_rzphi_to_xyz(xyz, rzphi)
#include "Implic.inc"
      REAL   xyz(*), rzphi(*)
c     transform from cylindrical to cartesian coordinates
      xyz(3) = rzphi(2)
      xyz(1) = rzphi(1) * cos(rzphi(3))
      xyz(2) = rzphi(1) * sin(rzphi(3))
      return
      END
c     .         --------------------------------------------------------
      SUBROUTINE X_xyz_norm(norm, xyz)
#include "Implic.inc"
      REAL   norm, xyz(*)
      norm = sqrt(xyz(1) * xyz(1) + xyz(2) * xyz(2) + xyz(3) *
     ^     xyz(3))
      return
      END
c     .         --------------------------------------------------------
      SUBROUTINE X_rzphi_norm(norm, rzphi)
#include "Implic.inc"
      REAL   norm, rzphi(*)
      norm = sqrt(rzphi(1) * rzphi(1) + rzphi(2) * rzphi(2))
      return
      END
c     .         --------------------------------------------------------
      SUBROUTINE X_det_33(det, mat)
#include "Implic.inc"
      REAL   det, mat(3, *)
      det = mat(1, 1) * mat(2, 2) * mat(3, 3) +
     ^     mat(1, 2) * mat(2, 3) * mat(3, 1) + mat(1, 3) *
     ^     mat(2, 1) * mat(3, 2) - mat(1, 1) * mat(3, 2) *
     ^     mat(2, 3) - mat(2, 1) * mat(1, 2) * mat(3, 3) -
     ^     mat(3, 1) * mat(2, 2) * mat(1, 3)
      return
      END
c     .         --------------------------------------------------------
      SUBROUTINE X_xyz_to_rzphi(rzphi, xyz)
#include "Implic.inc"
#include "numerics.inc"
      REAL   xyz(*), rzphi(*)
c     transform from cartesian to cylindrical coordinates
      rzphi(2) = xyz(3)
      rzphi(1) = sqrt(xyz(1) * xyz(1) + xyz(2) * xyz(2))
      if(rzphi(1) .eq. 0.)then
         rzphi(3) = 0.
      else if(xyz(1) .gt. 0.)then
         rzphi(3) = asin(xyz(2) / rzphi(1))
      else
         rzphi(3) = pi - asin(xyz(2) / rzphi(1))
      endif
      return
      END
c     .         --------------------------------------------------------
      SUBROUTINE X_setup_camera
#include "Implic.inc"
#include "emparams.inc"
#include "emitter.inc"
#include "camera.inc"
#include "numerics.inc"

      INTEGER ix, iy, i, j
      REAL   chi, mag_xy, del_R, del_z, rpy_sq(NPIXDIM),
     ^     FocLen2, axis_orientation(3, 3), 
     ^     chord_orientation(3), x_pix_min, x_pix_max, y_pix_min,
     ^     y_pix_max, rpx_sq, rp_sq, phi_pix, mu_pix, xyz_pinhole(3),
     ^     distance_pix_to_pinhole, det, norm, y_hat_xyz_pinhole(3)
      REAL    rt_rp_sq, asinarg, SMALL
      DATA    SMALL /1.0e-10/

      PI = 4. * atan(1.)
      R_bound_min_sq = R_bound_min * R_bound_min
      R_bound_max_sq = R_bound_max * R_bound_max

      call X_rzphi_to_xyz(xyz_pinhole, pinhole_loc)
c     
c     begin by:
c     determining orientation of camera symmetry axis relative to 
c     orientation of unit vector from pinhole (R, z, phi) to
c     tangency point prescribed by R_tangent, z_tangent
c     
c     set up camera axis orientation -- spherical angles
      call X_normvec_xyz(3, y_hat_xyz_pinhole, xyz_pinhole)
      call X_xyz_norm(norm, y_hat_xyz_pinhole)
      chi = phi_pinhole + acos(R_tangent / Rpinhole)
      del_R = Rpinhole * cos(pi / 2. - (chi - phi_pinhole))
      del_z = (z_tangent - Zpinhole)
      mag_xy = sqrt(del_R * del_R + del_z * del_z)
      phi_axis = pi / 2. + chi
      mu_axis = acos(del_z / mag_xy)
      axis_orientation(1, 3) = sin(mu_axis) * cos(phi_axis)
      axis_orientation(2, 3) = sin(mu_axis) * sin(phi_axis)
      axis_orientation(3, 3) = cos(mu_axis)
      axis_orientation(1, 2) = sin(phi_axis)
      axis_orientation(2, 2) = - cos(phi_axis)
      axis_orientation(3, 2) = 0.
      axis_orientation(1, 1) = - cos(phi_axis) * cos(mu_axis)
      axis_orientation(2, 1) = - sin(phi_axis) * cos(mu_axis)
      axis_orientation(3, 1) = sin(mu_axis)
      call X_det_33(det, axis_orientation)
c     gives the (x, y, z) projections of the three camera axes
c     
c     next:
c     set up to compute orientation (cartesian coordinates) of chords 
c     originating from each element 
c     
c     first step:
c     determine angular offset, relative to camera axis, of chords
c     eminating from each CCD element
c     orientation of axes is that positive x axis lies in vertical plane
c     formed by camera symmetry axis and tokamak centerline (z axis)
c     in camera coordinates then:
c     symmetry axis is polar axis, x axis is origin for phi and 
c     y = z cross vx 
c     the pixels are assumed located on the x-y plane, a distance 
c     focal_length from the pinhole, with dimensions 
c     pix_sixe_x and pix_size_y
      FocLen2 = focal_length * focal_length
      x_pix_min = - 0.5 * float(n_pixel_x) * pix_size_x
      x_pix_max = - x_pix_min
      y_pix_min = - 0.5 * float(n_pixel_y) * pix_size_y
      y_pix_max = - y_pix_min
      call ugrid(x_pix_vec, n_pixel_x, x_pix_min, x_pix_max)
      call ugrid(y_pix_vec, n_pixel_y, y_pix_min, y_pix_max)
      do iy = 1, n_pixel_y
         rpy_sq(iy) = y_pix_vec(iy) * y_pix_vec(iy)
      enddo
      do ix = 1, n_pixel_x
         rpx_sq = x_pix_vec(ix) * x_pix_vec(ix)
         do iy = 1, n_pixel_y
            rp_sq = rpx_sq + rpy_sq(iy)
            rt_rp_sq = sqrt(rp_sq) * ( 1.0 + SMALL )
c           mu_pix = asin(sqrt(rp_sq / (rp_sq + FocLen2)))
            asinarg = sqrt( rp_sq / (rp_sq + FocLen2) )
            mu_pix = asin(asinarg)
            if(rp_sq .eq. 0.)then
               phi_pix = 0.
            else if(x_pix_vec(ix) .ge. 0.)then
               asinarg = y_pix_vec(iy) / rt_rp_sq
c              call asinDebg(asinarg,ix,iy,n_pixel_x,n_pixel_y,2)
c              phi_pix = asin(y_pix_vec(iy) / sqrt(rp_sq))
               phi_pix = asin(asinarg)
            else
               asinarg = y_pix_vec(iy) / rt_rp_sq
c              call asinDebg(asinarg,ix,iy,n_pixel_x,n_pixel_y,3)
c              phi_pix = pi - asin(y_pix_vec(iy) / sqrt(rp_sq))
               phi_pix = pi - asin(asinarg)
            endif
c     generate the projection of chord orientation onto camera 
c     axes
            phi_pix = pi + phi_pix
c     chord originates at pixel plane
            chord_orientation(1) = sin(mu_pix) * cos(phi_pix)
            chord_orientation(2) = sin(mu_pix) * sin(phi_pix)
            chord_orientation(3) = cos(mu_pix)
            call X_xyz_norm(norm, chord_orientation)
            distance_pix_to_pinhole = sqrt(rp_sq + FocLen2)
            do i = 1, 3
               y_hat_camera(i, ix, iy) = 0.
               do j = 1, 3
                  y_hat_camera(i, ix, iy) = y_hat_camera(i, ix, iy) +
     ^                 axis_orientation(i, j) * chord_orientation(j)
c     in cartesian coordinates
               enddo
            enddo
            call X_xyz_norm(norm, y_hat_camera(1, ix, iy))
            do i = 1, 3
               chord_origin(i, ix, iy) = xyz_pinhole(i) -
     ^              distance_pix_to_pinhole * y_hat_camera(i, ix, iy)
c     in cartesian coordinates
            enddo
         enddo
      enddo
      return
      END
c     .         --------------------------------------------------------
c      SUBROUTINE asinDebg(asinarg,ix,iy,n_pixel_x,n_pixel_y,icall)
c      if (asinarg .gt. 1.0 ) then
c        write(6,'('' asinarg,ix,iy,npixx,npixy,icall:''
c     ^  e20.13,5i4)')asinarg,ix,iy,n_pixel_x,n_pixel_y,icall
c        asinarg=+1.00
c      endif
c      if (asinarg .lt.-1.0 ) then
c        write(6,'('' asinarg,ix,iy,npixx,npixy,icall:''
c     ^  e20.13,5i4)')asinarg,ix,iy,n_pixel_x,n_pixel_y,icall
c        asinarg=-1.00
c      endif
c      return
c      END
c     .         --------------------------------------------------------
      SUBROUTINE X_normvec_xyz(n, y_hat, y)
#include "Implic.inc"
      INTEGER n
      REAL   norm, y_hat(n), y(n), ynorminv
      call X_xyz_norm(norm, y)
      ynorminv = 1. / norm
      call X_vsmult(n, y_hat, y, ynorminv)
      return
      END
c     .         --------------------------------------------------------
      SUBROUTINE X_vsmult(n, vout, vin, c)
#include "Implic.inc"
      INTEGER n, i
      REAL   vout(n), vin(n), c
      do i = 1, n
         vout(i) = c * vin(i)
      enddo
      return
      END
c     .         --------------------------------------------------------
      SUBROUTINE X_diagnose_trajectories
#include "Implic.inc"
#include "emparams.inc"
#include "camera.inc"
      INTEGER ix, iy
      INTEGER pix_count(NPIXDIM)
      INTEGER iDBG
      DATA iDBG/0/
      if (iDBG .eq. 0 ) return
      do ix = 1, NPIXDIM
         pix_count(ix) = ix
      enddo
      write(4, 101)(pix_count(iy), pix_count(iy), iy = 1, n_pixel_y)
 101  format(10(i5, 1x, i5, 2x))
      write(4, 102)
 102  format(/)
      do ix = 1, n_pixel_x
            write(4, 100)(r_tangent_actual(ix, iy),
     ^           z_tangent_actual(ix, iy), iy = 1, n_pixel_y)
 100        format(10(f5.1, 1x, f5.1, 2x))
      enddo
      return
      END
c     .         --------------------------------------------------------
      SUBROUTINE X_compute_pattern
#include "Implic.inc"
#include "emparams.inc"
#include "camera.inc"
#include "numerics.inc"
      INTEGER ix, iy
      REAL   y_beg_rzphi(3), y_hat_xyz(3), y_end(3), SolAng
      solAng = 3.14159*(pinhole_size/focal_length)**2 * 0.25e+2
      do ix = 1, n_pixel_x
         do iy = 1, n_pixel_y
            call X_init_chord(ix, iy, y_beg_rzphi, y_hat_xyz)
            call X_c_phot(y_beg_rzphi, y_hat_xyz, dlxray, y_end, 3,
     ^           photon_count(ix, iy), npoints(ix, iy),
     ^           chord(1, 1, ix, iy), r_tangent_actual(ix, iy),
     ^           z_tangent_actual(ix, iy))
            photon_count(ix, iy) =  photon_count(ix, iy)*SolAng
         enddo
      enddo
c      call LSCpause
c      do ix = 1, n_pixel_x
c         do iy = 1, n_pixel_y
c
c        if (pc(ix,iy) .gt. 1.e+20 .or.
c     ^      pc(ix,iy) .lt.-1.e+20  ) then
c            write(6,'('' pc is bad at i,j;'', e10.3, i4, i4)')
c     ^      pc(ix,iy), ix, iy
c            pc(ix,iy) = 0.
c         endif
c         enddo
c      enddo
c
c      call LSCpause

      return
      END
c     .         --------------------------------------------------------
      SUBROUTINE X_init_chord(ix, iy, y_beg_rzphi, y_hat_xyz)
#include "Implic.inc"
#include "emparams.inc"
#include "camera.inc"
      INTEGER ix, iy
      REAL    y_beg_rzphi(*), y_hat_xyz(*)
      call X_xyz_to_rzphi(y_beg_rzphi, chord_origin(1, ix, iy))
      call xVecCopy(3, y_hat_xyz, y_hat_camera(1, ix, iy))
      return
      END
c     .         --------------------------------------------------------
      INTEGER FUNCTION inbound(rsq, rr, zz)
#include "Implic.inc"
#include "emparams.inc"
#include "emitter.inc"
      REAL   rsq, rr, zz
      if((rsq .ge. R_bound_min_sq) .and. (rsq .le. R_bound_max_sq)
     ^     .and.
     ^     (zz .ge. Z_bound_min) .and. (zz .le. Z_bound_max)
     ^     .and. 
     ^     ( (rr-PusherMajor)**2 + zz**2 .ge. PusherMinor**2) ) then
         inbound = 1
      else
         inbound = 0
      endif
      return
      END
c     .         --------------------------------------------------------
      SUBROUTINE X_plot_source_profile
#include "Implic.inc"
#include "emparams.inc"
#include "emitter.inc"
#include "camera.inc"
      INTEGER iz, ir
      REAL   delta_em, rr, zz
      call ugrid(r_source, nr_source, R_plasm_min, R_plasm_max)
      call ugrid(z_source, nz_source, Z_plasm_min, Z_plasm_max)
      do iz = 1, nz_source
         zz = z_source(iz)
         do ir = 1, nr_source
            rr = r_source(ir)
            source_profile(ir, iz) = delta_em(rr, zz, mu_0) 
         enddo
      enddo
      call X_pl_emisivity_cntrs(source_profile, NRDIM, nr_source,
     ^     nz_source,r_source,z_source)
      return
      END
c     .         --------------------------------------------------------
      SUBROUTINE X_plot_pixel_data
#include "Implic.inc"
#include "emparams.inc"
#include "emitter.inc"
#include "camera.inc"
      INTEGER nr, DOCHORDPLOT
      REAL    Radii(2), Z_height
      DATA DOCHORDPLOT / 0 /
c     plot chords in (x, y) and (r, z) views
      nr = 2
      Radii(1) = R_bound_min
      Radii(2) = R_bound_max
      Z_height = Z_bound_max
      if(DOCHORDPLOT .eq. 1 ) then
      call X_plan_view(nr, Radii, npoints, n_pixel_x, n_pixel_y, chord,
     ^     MAXPOINTS, NPIXDIM)
      call X_elevation_view(nr, Radii, - Z_height, Z_height, npoints,
     ^     n_pixel_x, n_pixel_y, chord, MAXPOINTS, NPIXDIM)
      endif
      call X_pl_intensity_cntrs(photon_count, NPIXDIM, n_pixel_x,
     ^     n_pixel_y)
      return
      END
c     .         --------------------------------------------------------
      SUBROUTINE xSlices
#include "Implic.inc"
#include "emparams.inc"
#include "emitter.inc"
#include "camera.inc"
c     DATA RcntChrd  / 1.65 / ! the nominal radius of the usual central
c                             ! chord plotted in the time development of the
c                             ! xray signal
      CHARACTER*70 MyString
      INTEGER i,j,icenter,jcenter, ioffset, joffset
      REAL    Tan2Pin, SHAVE 
      REAL    slice(NPIXDIM,2),
     ^        signal(NPIXDIM, NCHORDIM), sigMax, xmaxy
      DATA    SHAVE / 0.985 /
      REAL    ZERO, ONE
      DATA    ZERO, ONE /
     ^        0.0 , 1.0 /

      Tan2Pin = sqrt ( Rpinhole**2 - R_tangent**2 )
c23456789 123456789 123456789 123456789 123456789 123456789 123456789 123456789
      joffset = n_pixel_y * 
     ^  abs(RcntChrd-R_tangent) / Tan2Pin * focal_length / (screen_d)
      ioffset = n_pixel_x * 
     ^  abs(RcntChrd-R_tangent) / Tan2Pin * focal_length / (screen_d)
      icenter = (n_pixel_x + 1)/2
      jcenter = (n_pixel_y + 1)/2
c
c
      sigMax=0.00
      do 10 i=1, n_pixel_x
        slice(i,1) = float(i)
        signal(i,1) = photon_count(i,(jcenter))
        signal(i,2) = photon_count(i,(jcenter-joffset))
        signal(i,3) = photon_count(i,(jcenter-joffset/2))
        signal(i,4) = photon_count(i,(jcenter+joffset))
        signal(i,5) = photon_count(i,(jcenter+joffset/2))
        sigMax= amax1(sigMax, signal(i,1) )
        sigMax= amax1(sigMax, signal(i,2) )
        sigMax= amax1(sigMax, signal(i,3) )
        sigMax= amax1(sigMax, signal(i,4) )
        sigMax= amax1(sigMax, signal(i,5) )
 10   continue
      do 11 j=1, n_pixel_y
        slice(j,2) = float(j)
        signal(j,6) = photon_count(icenter,j)
        sigMax= amax1(sigMax, signal(j,6))
 11   continue
      call XRopen(8,'XRinput')
      call XRdscw(slice(1,1), signal(1,2), n_pixel_x,
     ^            'Vertical x ray slice')
      call XRclos(8)

      do 15 i=1, n_pixel_x
        signal(i,1) = signal(i,1)/(sigMax+1.e-30)*SHAVE
        signal(i,2) = signal(i,2)/(sigMax+1.e-30)*SHAVE
        signal(i,3) = signal(i,3)/(sigMax+1.e-30)*SHAVE
        signal(i,4) = signal(i,4)/(sigMax+1.e-30)*SHAVE
        signal(i,5) = signal(i,5)/(sigMax+1.e-30)*SHAVE
 15   continue
      do 16 j=1, n_pixel_y
        signal(j,6) = signal(j,6)/(sigMax+1.e-30)*SHAVE
 16   continue
c
c     ULH corner
      call EZrnd2 (float(n_pixel_x),xmaxy,i,j)
      call EZsets(100,500,450,700,
     ^ ZERO,xmaxy,ZERO, ONE, 1)
c    ^ 0.,xmaxy,0.00, 1.00, 1)
      call EZaxes(1,2,1,5)
      call EZcurv(slice(1,1),signal(1,1),n_pixel_x)
      write(MyString,'(''Vert slice /'',1pe9.2, 
     ^    '' at tangent$'')') sigMax
      call EZwrit(100,725,MyString,0,0)
c     URH corner
      call EZsets(600,1000,450,700,
     ^ ZERO,xmaxy,ZERO, ONE, 1)
c    ^ 0.,xmaxy,0.00, 1.00, 1)
      call EZaxes(1,2,1,5)
      call EZcurv(slice(1,1),signal(1,2),n_pixel_x)
      call EZcros(slice(1,1),signal(1,3),n_pixel_x)

      write(MyString, '(''at chord'',i3,'' and'',i3,'' (dots)$'')') 
     ^                   (jcenter-joffset), (jcenter-joffset/2)
      call EZwrit(600,725,MyString,0,0)
c
c     LLH corner
      call EZsets(100,500,100,350,
     ^ ZERO,xmaxy,ZERO, ONE, 1)
c    ^ 0.,xmaxy,0.00, 1.00, 1)
      call EZaxes(1,2,1,5)
      call EZcurv(slice(1,1),signal(1,4),n_pixel_x)
      call EZcros(slice(1,1),signal(1,5),n_pixel_x)
      write(MyString, '(''at chord'',i3,'' and'',i3,'' (dots)$'')') 
     ^                   (jcenter+joffset), (jcenter+joffset/2)
      call EZwrit(100,375,MyString,0,0)
c     
c     LRH corner
      call EZrnd2 (float(n_pixel_y),xmaxy,i,j)
      call EZsets(600,1000,100,350,
     ^ ZERO,xmaxy,ZERO, ONE, 1)
c    ^ 0.,xmaxy,0.00, 1.00, 1)
      call EZaxes(1,2,1,5)
      call EZcurv(slice(1,2),signal(1,6),n_pixel_y)
      write(MyString,'(''Horiz slice '',''at mid plane$'')') 
      call EZwrit(600,375,MyString,0,0)
c
           call MkGrfLst('X ray slices; 4 graphs')
           call EZfini(0,0)
c
c
      return
      END
c     .         --------------------------------------------------------
      SUBROUTINE eSlices
#include "Implic.inc"
#include "emparams.inc"
#include "emitter.inc"
c#include "camera.inc"
      CHARACTER*70 MyString
      INTEGER i,j,icenter,jcenter, ioffset, joffset
      REAL    SHAVE 
      REAL    signal(NRZMXDIM, NCHORDIM), sigMax
      REAL    rminy, rmaxy, rdumy, zmaxy
      DATA    SHAVE / 0.985 /
      REAL    ZERO, ONE
      DATA    ZERO, ONE /
     ^        0.0 , 1.0 /

      ioffset = nr_source / 4
      joffset = nz_source / 4
      icenter = (nr_source + 1)/2
      jcenter = (nz_source + 1)/2
c
c
      sigMax=0.00
      do 10 j=1, nz_source
        signal(j,1) = source_profile((icenter          ),j)
        signal(j,2) = source_profile((icenter+ioffset  ),j)
        signal(j,3) = source_profile((icenter+ioffset/2),j)
        signal(j,4) = source_profile((icenter-ioffset  ),j)
        signal(j,5) = source_profile((icenter-ioffset/2),j)
        sigMax= amax1(sigMax, signal(j,1) )
        sigMax= amax1(sigMax, signal(j,2) )
        sigMax= amax1(sigMax, signal(j,3) )
        sigMax= amax1(sigMax, signal(j,4) )
        sigMax= amax1(sigMax, signal(j,5) )
 10   continue
      do 11 i=1, nr_source
        signal(i,6) = source_profile(i,jcenter)
        sigMax= amax1(sigMax, signal(i,6))
 11   continue

      do 15 j=1, nz_source
        signal(j,1) = signal(j,1)/(sigMax+1.e-30)*SHAVE
        signal(j,2) = signal(j,2)/(sigMax+1.e-30)*SHAVE
        signal(j,3) = signal(j,3)/(sigMax+1.e-30)*SHAVE
        signal(j,4) = signal(j,4)/(sigMax+1.e-30)*SHAVE
        signal(j,5) = signal(j,5)/(sigMax+1.e-30)*SHAVE
 15   continue
      do 16 i=1, nr_source
        signal(i,6) = signal(i,6)/(sigMax+1.e-30)*SHAVE
 16   continue
c
c     ULH corner
      call EZrnd2( z_source(nz_source),zmaxy,i,j )
      call EZsets(100,500,450,700,
     ^ -zmaxy,zmaxy,ZERO, ONE, 1)
c    ^ -zmaxy,zmaxy,0.00, 1.00, 1)
      call EZaxes(i,j,1,5)
      call EZcurv(z_source,signal(1,1),nz_source)
      write(MyString,'(''Vert slice /'',1pe9.2, 
     ^    '' at center$'')') sigMax
      call EZwrit(100,725,MyString,0,0)
c
c     URH corner
      call EZsets(600,1000,450,700,
     ^ -zmaxy,zmaxy,ZERO,  ONE, 1)
c    ^ -zmaxy,zmaxy,0.00, 1.00, 1)
      call EZaxes(i,j,1,5)
      call EZcurv(z_source,signal(1,2),nz_source)
      call EZcros(z_source,signal(1,3),nz_source)

      write(MyString, '(''at r_loc'',i3,'' and'',i3,'' (dots)$'')') 
     ^                   (icenter+ioffset), (icenter+ioffset/2)
      call EZwrit(600,725,MyString,0,0)
c
c     LLH corner
      call EZsets(100,500,100,350,
     ^ -zmaxy,zmaxy,ZERO,  ONE, 1)
c    ^ -zmaxy,zmaxy,0.00, 1.00, 1)
      call EZaxes(i,j,1,5)
      call EZcurv(z_source,signal(1,4),nz_source)
      call EZcros(z_source,signal(1,5),nz_source)
      write(MyString, '(''at r_loc'',i3,'' and'',i3,'' (dots)$'')') 
     ^                   (icenter-ioffset), (icenter-ioffset/2)
      call EZwrit(100,375,MyString,0,0)
c     
c     LRH corner
      call EZrnd2 (r_source(nr_source), rmaxy, i,j)
      call EZrnd2 (rmaxy-r_source(1),rdumy, i,j)
      rminy = rmaxy - rdumy
      call EZsets(600,1000,100,350,
     ^ rminy,rmaxy,ZERO,  ONE, 1)
c    ^ rminy,rmaxy,0.00, 1.00, 1)
      call EZaxes(i,j,1,5)
      call EZcurv(r_source,signal(1,6),nr_source)
      write(MyString,'(''Horiz slice '',''at mid plane$'')') 
      call EZwrit(600,375,MyString,0,0)
c
           call MkGrfLst('Emissivity slices; 4 graphs')
           call EZfini(0,0)
c
      return
      END
c     .         --------------------------------------------------------
c      SUBROUTINE XmnGraf
c#include "Implic.inc"
c#include "xparams.inc"
c#include "Xray.inc"

c...
c      return
c      END
c     .         --------------------------------------------------------
      SUBROUTINE X_wrres
#include "Implic.inc"
#include "params.inc"
#include "emparams.inc"
#include "xparams.inc"
#include "Xray.inc"
#include "emitter.inc"
#include "camera.inc"
c     write results to disk file

      INTEGER ix, iy
      INTEGER iDBG
      DATA iDBG/1/
      if (iDBG .eq. 0 ) return
c.
      write(4, 102) 'photon_c'
 102  format(/,a10)
      do ix = 1, NPIXDIM
      write(4, 101)(photon_count(ix,iy), iy = 1, NPIXDIM)
 101  format(8(1pe10.1))
      enddo
      write(4, 102)
c.
      write(4, 102) 'source_p'
      do ix = 1, NRDIM
      write(4, 101)(source_profile(ix,iy), iy = 1, NZDIM)
      enddo
      write(4, 102)
c.
      write(4, 102) 'sigtot'
      do ix = 1, NMUDIM
      write(4, 101)(sigtot(ix,iy), iy = 1, NENDIM)
      enddo
      write(4, 102)
c.
      write(4, 102) 'inten'
      do ix = 1, NMUDIM
      write(4, 101)(inten(ix,iy), iy = 1, NPSIDIM)
      enddo
      write(4, 102)
c.

      return
      END
c     .         --------------------------------------------------------      
      SUBROUTINE X_c_phot(y_beg, y_hat, dlchord, y_end, n_space, pc,
     ^     npoints, chord, r_tangent_actual, z_tangent_actual)
#include "Implic.inc"
#include "emparams.inc"
#include "numerics.inc"
      INTEGER n_space, inbound, i, count, state,
     ^     current_status, previous_status
c     INTEGER*4 npoints
      INTEGER   npoints
      REAL   
     ^     y_beg(n_space), dlchord, y(SPACEDIM), y_hat(SPACEDIM),
     ^     delta_em, y_end(n_space), mu_yb, dy_xyz(3),
     ^     r_tangent_actual, z_tangent_actual, radius_sq,
     ^     radius_sq_min, rr, zz
c     REAL*4 chord(3, *), pc
      REAL   chord(3, *), pc
      call X_rzphi_to_xyz(y, y_beg)
      do i = 1, 3
         dy_xyz(i) = y_hat(i) * dlchord
      enddo
      pc = 0.
      count = 0
      state = 1
      radius_sq = y(1) * y(1) + y(2) * y(2)
      rr = sqrt (radius_sq)
      radius_sq_min = radius_sq
      previous_status = inbound(radius_sq, rr , y(3))
      current_status = previous_status
      do while((state .lt. 3) .and. (count .lt. npoints_max))
         do i = 1, 3
            chord(i, count + 1) = y(i)
            y(i) = y(i) + dy_xyz(i)
         enddo
         radius_sq = y(1) * y(1) + y(2) * y(2)
         rr = sqrt (radius_sq)
         if(radius_sq_min .gt. radius_sq)then
            radius_sq_min = radius_sq
            z_tangent_actual = y(3)
         endif
         count = count + 1
         if(state .eq. 2)then
c           rr = sqrt(radius_sq) ! we already did this
            zz = y(3)
            pc = pc + delta_em(rr, zz, mu_yb(y, y_hat)) * dlchord 
         endif
         previous_status = current_status
         current_status = inbound(radius_sq, rr, y(3))
         if(current_status .ne. previous_status)  then
            state = state + 1
         endif
      enddo
      r_tangent_actual = sqrt(radius_sq_min)
      npoints = count
      call X_xyz_to_rzphi(y_end, y)
      return
      END
c     .         --------------------------------------------------------      
      SUBROUTINE X_advance_y(y, dy_xyz)
#include "Implic.inc"
      INTEGER i
      REAL   y(*), dy_xyz(*), y_old(3)
      call X_rzphi_to_xyz(y_old, y)
c     call X_project_dy(y, dy_xyz, dy_rzphi)
      do i = 1, 3
         y(i) = y_old(i) + dy_xyz(i)
      enddo
      return
      END
c     .         --------------------------------------------------------
      SUBROUTINE X_project_dy(y_rzphi, dy_xyz, dy_rzphi)
#include "Implic.inc"
#include "emparams.inc"
#include "numerics.inc"
c     given y (R, z, phi) and dy (dx, dy, dz), compute (dR, dz, dphi)
      INTEGER i, j
      REAL   y_rzphi(*), dy_xyz(*), dy_rzphi(*), T(3, 3), dRdx, dRdy,
     ^     dRdz, dzdx, dzdy, dzdz, dphidx, dphidy, dphidz,
     ^     xx, yy, Rsq_inv
      EQUIVALENCE
     ^     (T(1, 1), dRdx), (T(1, 2), dRdy), (T(1, 3), dRdz),
     ^     (T(2, 1), dzdx), (T(2, 2), dzdy), (T(2, 3), dzdz),
     ^     (T(3, 1), dphidx), (T(3, 2), dphidy), (T(3, 3), dphidz)
      DATA dRdz / 0. / dzdx, dzdy, dzdz / 0, 0, 1. / dphidz / 0. /
      xx = y_rzphi(1) * cos(y_rzphi(3))
      yy = y_rzphi(1) * sin(y_rzphi(3))
      dRdx = xx / y_rzphi(1)
      dRdy = yy / y_rzphi(1)
      Rsq_inv = 1. / (y_rzphi(1) * y_rzphi(1))
      dphidx = - yy * Rsq_inv
      dphidy = xx * Rsq_inv
      do i = 1, 3
         dy_rzphi(i) = 0.
         do j = 1, 3
            dy_rzphi(i) = dy_rzphi(i) + T(i, j) * dy_xyz(j)
         enddo
      enddo
      return
      END
c     .         --------------------------------------------------------
      REAL   FUNCTION mu_yb(y, y_hat)
#include "Implic.inc"
      INTEGER i
      REAL   y(*), y_hat(*), inner_prod, B_vector(3), rr
c     B(1) = B_x, B(2) = B_y, B(3) = B_z
c     construct cos angle between magnetic field (assumed in phi direction)
c     and y_hat at point y (in r, Z, phi, coordinates) and
c     unit vector y_hat (in cartesian x, y, z, coordinates)
      rr = sqrt(y(1) * y(1) + y(2) * y(2))
      B_vector(1) = y(2) / rr
      B_vector(2) = - y(1) / rr
      B_vector(3) = 0.
      inner_prod = 0.
      do i = 1, 3
         inner_prod = inner_prod + B_vector(i) * y_hat(i)
      enddo
      mu_yb = inner_prod
      return
      END
c     .         --------------------------------------------------------
      SUBROUTINE xVecCopy(n, tovec, frvec)
#include "Implic.inc"
      INTEGER n, i
      REAL    tovec(n), frvec(n)
      do i = 1, n
         tovec(i) = frvec(i)
      enddo
      return
      END
c     .         --------------------------------------------------------
c
c     graphs    --------------------------------------------------------
      SUBROUTINE X_plan_view(nr, Radii, npoints,nx,ny, chord4, n1,n34)
#include "Implic.inc"
      INTEGER n34, nx, ny, nr, n1
c     INTEGER*4 npoints(n34, n34)
      INTEGER   npoints(n34, n34)
      REAL    R_min, R_max, Radii(nr)
c     REAL*4  chord4(3, n1, n34, n34)
      REAL    chord4(3, n1, n34, n34)
      call VecMnMx( Radii, nr, R_min, R_max)
      call xPlanSet (R_min, R_max)
      call PlPlBnds(nr, Radii)
      call PlPlChrds (npoints, nx, ny, chord4, n1, n34)
      return
      END
c     .         --------------------------------------------------------
      SUBROUTINE xPlanSet (R_min, R_max)
#include "Implic.inc"
c     INTEGER*4 I1, I2
c     DATA      I1, I2 / 1, 2 /
      REAL   R_min, R_max
c     REAL*4 xdum4(2), ydum4(2), twormax
      REAL   twormax


      twormax = R_max + R_max
cNCAR      call agsetf('X/MINIMUM.', - twormax)
cNCAR      call agsetf('X/MAXIMUM.', twormax)
cNCAR      call agsetf('Y/MINIMUM.', - twormax)
cNCAR      call agsetf('Y/MAXIMUM.', twormax)
cNCAR      call agstup(xdum4, 1, 1, 2, 1, ydum4, 1, 1, 2, 1)
cNCAR      call agback 
           call EZsets (50,550,  50,550, 
     ^        -twormax, twormax, -twormax, twormax, 1)
      return
      END
c     .         --------------------------------------------------------
      SUBROUTINE PlPlBnds(nr, Radii)
#include "Implic.inc"
#include "xgraphs.inc"
c     INTEGER*4 nthet4, I1
      INTEGER   nthet4, I1
      INTEGER   i, nr, ir
      REAL      Radii(nr)
c     REAL*4  xva4(NWKDIX), yva4(NWKDIX), 
c    ^        costhet4(NWKDIX), sinthet4(NWKDIX)
      REAL    xva4(NWKDIX), yva4(NWKDIX), 
     ^        costhet4(NWKDIX), sinthet4(NWKDIX)
c     REAL*4    dthet4, pi, rim1, thet4
      REAL      dthet4, pi, rim1, thet4
      EQUIVALENCE (xva4(1), wkarx(1, 1)), (yva4(1), wkarx(1, 2)),
     ^     (costhet4(1), wkarx(1, 3)), (sinthet4(1), wkarx(1, 4))
      DATA I1/ 1 /
      nthet4 = NWKDIx 
      pi = 4. * atan(1.)
      dthet4 = (2. * pi) / float(nthet4 - 1)
      do i = 1, nthet4
         rim1 = float(i - 1)
         thet4 = dthet4 * rim1
         costhet4(i) = cos(thet4)
         sinthet4(i) = sin(thet4)
      enddo
      do ir = 1, nr
         do i = 1, nthet4
            xva4(i) = Radii(ir) * costhet4(i)
            yva4(i) = Radii(ir) * sinthet4(i)
         enddo
cNCAR         call agcurv(xva4, 1, yva4, 1, nthet4, 1)
              call EZcurv(xva4, yva4, nthet4)
      enddo
      return
      END
c     .         --------------------------------------------------------
      SUBROUTINE PlPlChrds(npoints, nx, ny, chord4, n1, n34)
#include "Implic.inc"
      INTEGER   n1, n34
      INTEGER   npoints(n34, n34), nx, ny, ix, iy, np
      REAL      chord4(3, n1, n34, n34)

      do ix = 1, nx, 2
        do iy = 1, ny, 2
cncar     call agcurv(chord4(1, 1, ix, iy), 3, chord4(2, 1, ix, iy),3, 
cncar^    npoints(ix, iy), 1)
c         call EZcurvmd(chord4(1, 1, ix, iy), 3, chord4(2, 1, ix, iy),3, 
c    ^    npoints(ix, iy))

          np = npoints(ix, iy) - 1

          call EZdra(chord4(1, 1, ix, iy), chord4(2, 1, ix, iy), 0 ) 
          call EZdra(chord4(1, 1, ix, iy), chord4(2, 1, ix, iy), 1 ) 
          call EZdra(chord4(1,np, ix, iy), chord4(2,np, ix, iy), 1 ) 

        enddo
      enddo
      return
      END
c     .         --------------------------------------------------------
      SUBROUTINE X_elevation_view(nr, Radii, Z_min, Z_max, npoints, nx,
     ^     ny, chord4, n1, n34)
#include "Implic.inc"
      INTEGER   nr, n34, nx, ny, n1
      INTEGER*4 npoints(n34, n34)
      REAL   R_min, R_max, Radii(nr), Z_min, Z_max
c     REAL*4 chord4(3, n1, n34, n34)
      REAL   chord4
      REAL    ZERO
      DATA    ZERO /
     ^        0.0  /
      call VecMnMx ( Radii, nr,  R_min, R_max)
c     call xElevSet (0., R_max, Z_min, Z_max)
      call xElevSet (ZERO, R_max, Z_min, Z_max)
      call PlElBnds (R_min, R_max, Z_min, Z_max)
      call PlElChrds (npoints, nx, ny, chord4, n1, n34)
           call MkGrfLst('Xray plan, elevation chords')
           call EZfini(0,0)
      return
      END
c     .         --------------------------------------------------------
      SUBROUTINE PlElBnds (R_min, R_max, Z_min, Z_max)
#include "Implic.inc"
c     INTEGER*4 i, I1, I2, I3, I4, I5
      INTEGER   i, I1, I2, I3, I4, I5
      REAL    R_min, R_max, Z_min, Z_max, ris
c     REAL*4  xva4 (5, 2), yva4 (5, 2)
      REAL    xva4 (5, 2), yva4 (5, 2)
      DATA I1, I2, I3, I4, I5 / 1, 2, 3, 4, 5 /
      REAL    ZERO
      DATA    ZERO /
     ^        0.0  /

c     i = 1: right half
c     i = 2: left half
      do i = I1, I2
         if(i .eq. I1) then
            ris = 1.
         else
            ris = -1.
         endif
         xva4(I1, i) = ris * R_min
         yva4(I1, i) = Z_min
         xva4(I2, i) = ris * R_max
         yva4(I2, i) = Z_min
         xva4(I3, i) = ris * R_max
         yva4(I3, i) = Z_max
         xva4(I4, i) = ris * R_min
         yva4(I4, i) = Z_max
         xva4(I5, i) = xva4(I1, i)
         yva4(I5, i) = yva4(I1, i)

cncar      call agcurv  (xva4(I1, i),I1, yva4(I1, i),I1,I5,I1)
           call EZcurvmd(xva4(I1, i),I1, yva4(I1, i),I1,I5)
           
      enddo
c          call EZdra(0.00, Z_min    , 0)
c          call EZdra(0.00, Z_min    , 1)
c          call EZdra(0.00, Z_min/10., 1)
c          call EZdra(0.00, 0.00     , 0)
c          call EZdra(0.00, 0.00     , 1)
c          call EZdra(0.00, Z_max/10., 0)
c          call EZdra(0.00, Z_max/10., 1)
c          call EZdra(0.00, Z_max    , 1)

           call EZdra(ZERO, Z_min    , 0)
           call EZdra(ZERO, Z_min    , 1)
           call EZdra(ZERO, Z_min/10., 1)
           call EZdra(ZERO, ZERO     , 0)
           call EZdra(ZERO, ZERO     , 1)
           call EZdra(ZERO, Z_max/10., 0)
           call EZdra(ZERO, Z_max/10., 1)
           call EZdra(ZERO, Z_max    , 1)

      return
      END
c     .         --------------------------------------------------------
      SUBROUTINE xElevSet (R_min_pl, R_max, Z_min, Z_max)
#include "Implic.inc"
c     INTEGER*4 I1, I2
      INTEGER   I1, I2
      REAL   R_min_pl, R_max, Z_min, Z_max, Z_mn, Z_mx
c     REAL*4 xdum4(2), ydum4(2), twormax, twozmin, twozmax
c     REAL   xdum4(2), ydum4(2)
      REAL   twormax, twozmin, twozmax
c     REAL*4 R_min_pl4
      REAL   R_min_pl4
      DATA I1, I2 / 1,2 /

      Z_mn = Z_min - 0.5 * (Z_max - Z_min)
      Z_mx = Z_max + 0.5 * (Z_max - Z_min)

      R_min_pl4 = R_min_pl

      twormax = 2. * R_max
      twozmin = 2. * Z_mn
      twozmax = 2. * Z_mx
      

cNCAR      call agsetf('X/MINIMUM.', R_min_pl4)
cNCAR      call agsetf('X/MAXIMUM.', twormax)
cNCAR      call agsetf('Y/MINIMUM.', twozmin)
cNCAR      call agsetf('Y/MAXIMUM.', twozmax)
cncar      call agstup(xdum4,I1,I1,I2,I1, ydum4,I1,I1,I2,I1)
cNCAR      call agback
           call EZsets (700,1000, 150,450, 
     ^        R_min_pl4, twormax, twozmin, twozmax, 1)
      return
      END
c     .         --------------------------------------------------------
      SUBROUTINE PlElChrds (npoints, nx, ny, chord4, n1, n34)
#include "Implic.inc"
#include "xgraphs.inc"
      INTEGER   n34, nx, ny, ix, iy, n1, ip, np
c     INTEGER*4 npoints(n34, n34), I1
      INTEGER   npoints(n34, n34), I1
c     REAL*4 chord4(3, n1, n34, n34)
      REAL   chord4(3, n1, n34, n34)
c     REAL*4 rr(1), zz(1)
      REAL   rr(NWKDIx), zz(NWKDIx)
      EQUIVALENCE (rr(1), wkarx(1, 1)), (zz(1), wkarx(1, 2))
      DATA I1 / 1 /
      do ix = 1, nx, 4
         do iy = 1, ny, 4
            do ip = 1, npoints(ix, iy)
               call xyz2rz4(chord4(1, ip, ix, iy),I1, rr(ip), zz(ip))
            enddo
cncar            call agcurv  (rr,I1, zz,I1, npoints(ix, iy),I1)
c                call EZcurvmd(rr,I1, zz,I1, npoints(ix, iy))

            np = npoints(ix, iy) - 1

            call EZdra(rr( 1), zz( 1), 0 ) 
            do ip=1,np,30
              call EZdra(rr(ip), zz(ip), 1 ) 
            enddo
         enddo
      enddo
      return
      END
c     .         --------------------------------------------------------
      SUBROUTINE xyz2rz4(xyz, n, rr, zz)
#include "Implic.inc"
      INTEGER n, i
      REAL    xyz(3, n)
c     REAL*4  rr(n), zz(n)
      REAL    rr(n), zz(n)
      do i = 1, n
         rr(i) = sqrt(xyz(1, i) * xyz(1, i) + xyz(2, i) * xyz(2, i))
         zz(i) = xyz(3, i)
      enddo
      return
      END
c     .         --------------------------------------------------------
      SUBROUTINE X_pl_intensity_cntrs(pc, nxdim, nx, ny)
#include "Implic.inc"
#include "emparams.inc"
      CHARACTER*5 whichside
      CHARACTER*30 ChStr
      INTEGER   nxdim, nx, ny, iside, ix1(2), ix2(2), iy1, iy2
c     REAL*4 pc (nxdim, ny)
      REAL   pc (nxdim, ny)
c.
      REAL    pcmin, pcmax, pcave 
      REAL    xminy, xmaxy, xdumy, yminy, ymaxy
      REAL    xAry(NPIXDIM), yAry(NPIXDIM), clevelin(20)
      REAL    xGiv(*)      , yGiv(*)
      INTEGER i,j, kclev1, kclev2
      INTEGER ixmax,ixmin,ixstep
      INTEGER jymax,jymin,jystep
      DATA    ix1(1), ix2(1), ix1(2), ix2(2), iy1, iy2 /
     ^           100,  500,    600,    1000 , 200, 600 /
      whichside = 'LEFT'
      iside = 1
      do 10 i=1,nx
        xAry(i) = float(i)
 10   continue
      do 11 j=1,ny
        yAry(j) = float(j)
 11   continue
      call EZrnd2 (xAry(nx)  , xmaxy, i,j)
      call EZrnd2 (yAry(ny)  , ymaxy, i,j)
      xminy = 0.00
      yminy = 0.00

      kclev1 = -5
      kclev2 = 2

c
      goto 15
      ENTRY      X_pl_emisivity_cntrs(pc, nxdim, nx, ny, 
     ^           xGiv, yGiv)
      whichside = 'RIGHT'
      iside = 2

c  round up from max found--> nice value; #major divns; #minor divns
      call EZrnd2 (xGiv(nx)  , xmaxy, i,j)
      call EZrnd2 (xmaxy-xGiv(1), xdumy, i,j)
      call EZrnd2 (yGiv(ny)  , ymaxy, i,j)
      if (2.*ymaxy .gt. xdumy) then
        xminy = xmaxy - 2.0*ymaxy
        yminy = - ymaxy 
      else
        xminy = xmaxy - xdumy
        ymaxy = xdumy / 2.
        yminy =-ymaxy
      endif

      kclev1 = -5
      kclev2 = 2
c
c     ...................
c
 15   continue

      pcmin = pc(1,1)
      pcmax = pc(1,1)
      pcave = 0.00
      do 20 i=1,nx
      do 19 j=1,ny
        pcmin = amin1(pcmin, pc(i,j))
        pcmax = amax1(pcmax, pc(i,j))
        pcave = pcave +      pc(i,j)
 19   continue
 20   continue
      pcave = pcave / float(nx *  ny)

c
c     NCAR Code STARTS:
c     Force PLOTCHAR to use characters of the lowest quality.
cNCAR      call pcseti ('QU - QUALITY FLAG',2)
c     initialize the drawing of the contour plot.
cNCAR      call cprect (pc, nxdim, nx, ny, wkarx, NWKDIX * 4,
cNCAR     ^     iwkarx, NWKDIX)
c     Draw the default background.
cNCAR      call cpback (pc, wkarx, iwkarx)
c     Draw contour lines and labels.
cNCAR      call cpcldr (pc, wkarx, iwkarx)
c     Add the informational label and the high/low labels.
cNCAR      call cplbdr (pc, wkarx, iwkarx)
c     Compute and print statistics for the plot, label it, and put a
c     boundary line around the edge of the plotter frame.
c     cALL CAPSAP ('counts', TIME, IAMA, 0)
c     cALL LABTOP ('EXAMPLE 1-1',.017)
c     cALL BNDARY
c     NCAR Code ENDS:
c     
c     ------------------------------------------------------------------
c     kclev1: >0 --> clevelin contains levels to be used; 
c                    dots for index less than kclev2; 
c                    solid for index greater or equal kclev2
c     kclev1: =0 --> first contour at clevelin(1) with
c                    next one up by clevelin(2), and so on and so on
c     kclev1: <0 --> rcontr to choose -kclev1 equally spaced values between
c                    clevelin(1) and clevelin(2)
c     clevelin:      array of contour levels; this is output if kclev1<0
c     kclev2:        separates dots from solid lines
c     ------------------------------------------------------------------
c      previous to Mar 93 usage:
c      kclev1 =-10
c      kclev2 = 2
c      pcmin = pcmin + (pcmax-pcmin) / 10.
c      clevelin(1) = pcmin
c      clevelin(2) = pcmax-pcmin


      pcmin = pcmax / 100.
      clevelin(1) = pcmin
      clevelin(2) = pcmax*0.91

      ixmin = 1
      ixmax = nx
      jymin = 1
      jymax = ny
      ixstep= 1
      jystep= 1

      if (iside .eq. 1) then
      call EZrcon(ix1(iside),ix2(iside),     iy1,        iy2,
     ^                 xminy,     xmaxy,   yminy,      ymaxy, 
     ^  kclev1,clevelin,kclev2,
     ^  pc,
     ^  nxdim,
     ^  xAry, ixmin, ixmax, ixstep,      
     ^  yAry, jymin, jymax, jystep )     
      else if (iside .eq. 2) then
      call EZrcon(ix1(iside),ix2(iside),     iy1,        iy2,
     ^                 xminy,     xmaxy,   yminy,      ymaxy, 
     ^  kclev1,clevelin,kclev2,
     ^  pc,
     ^  nxdim,
     ^  xGiv, ixmin, ixmax, ixstep,      
     ^  yGiv, jymin, jymax, jystep )     
      else
         continue
      endif
c
      if (iside .eq. 1)
     ^      write(ChStr,'(''min: '',1pe9.2,'' w/cm^2 ...$'')')pcmin
      if (iside .eq. 2)
     ^      write(ChStr,'(''min: '',1pe9.2,'' w/cm^3 ...$'')')pcmin
      call EZwrit(ix1(iside), 750, ChStr, 0,0)
c
      write(ChStr,'(''max: '',1pe9.2,''  $'')')pcmax
      call EZwrit(ix1(iside), 725, ChStr, 0,0)
c
      write(ChStr,'(''ave: '', 1pe9.2,'' $'')')pcave
      call EZwrit(ix1(iside), 700, ChStr, 0,0)

      if (iside .eq. 1) then
      write(ChStr,'(''Xmx/mn:'',2(1x,1pe9.2),''$'')')
     ^               xAry(ixmax),xary(1)
      call EZwrit(ix1(iside), 675, ChStr, 0,0)
      write(ChStr,'(''Ymx/mn:'',2(1x,1pe9.2),''$'')')
     ^               yAry(jymax),yary(1)
      call EZwrit(ix1(iside), 650, ChStr, 0,0)
      else if (iside .eq. 2 ) then
      write(ChStr,'(''Xmx/mn:'',2(1x,1pe9.2),''$'')')
     ^               xGiv(ixmax),xGiv(1)
      call EZwrit(ix1(iside), 675, ChStr, 0,0)
      write(ChStr,'(''Ymx/mn:'',2(1x,1pe9.2),''$'')')
     ^               yGiv(jymax),yGiv(1)
      call EZwrit(ix1(iside), 650, ChStr, 0,0)

      endif

c     
      return
      END
c
c     +---------------------------------------------------------------+
c
      SUBROUTINE xSigGen(Emev, Ephoton, mu, sigma)
#include "Implic.inc"
      REAL   EMev, Ephoton, mu, sigma, T0, k
      REAL   signorm, Mevpermcsq
      COMMON / sigbrcom0 / signorm, Mevpermcsq
c      
c     converts to / from Mev to Koch, Motz units 
c     input energies in Mev, angle in radians
c     return sigma in cm**2 / sR / MeV
c
      T0 = Emev / Mevpermcsq
      k = Ephoton / Mevpermcsq
      call xSigBrem(T0, k, mu, sigma)
      sigma = sigma / Mevpermcsq
      return
      END
      
c     +---------------------------------------------------------------+
      
      SUBROUTINE xBremSetup(Z)
#include "Implic.inc"
      
      REAL   Z
      REAL   pie_local, r0, ECHARGE, EMASS, CLIGHT, HBAR, alpha
      PARAMETER(ECHARGE = 4.8e-10, EMASS = 9.1e-28,
     ^     CLIGHT = 3.e10, HBAR = 1.05e-27)
      REAL   signorm, Mevpermcsq
      COMMON / sigbrcom0 / signorm, Mevpermcsq
c     
      MevPerMcsq = .511
      pie_local = 4. * atan(1.)
      alpha = ECHARGE * ECHARGE / (HBAR * CLIGHT)
      r0 = ECHARGE * ECHARGE / (EMASS * CLIGHT * CLIGHT)
      signorm = alpha * r0 * r0 * Z * Z / ( 8. * pie_local )
      return
      END

c     +-----------------------------------------------------------------+

      SUBROUTINE xSigBrem(T0, k, mu, sigma)
#include "Implic.inc"
c     Reference:
c     H. W. Koch and J. W. Motz, 
c     ``Bremsstrahlung Cross-Section Formulas and Related Data,''
c     \RMP{31}{920}{59}.
c     Formula 2BN on p 924
c     '2' denotes differential in photon energy and angle
c     'B' denotes Born approximation
c     'N' denotes no screening
c     T0       INPUT  initial k-e of electron in a collision; m_0 c^2 units
c     k        INPUT  energy of emitted photon;               m_0 c^2 units
c     mu       INPUT  cos theta_0; angle between initial e-momentum and emitted
c                     photon
c     sigma    OUTPUT differential cross section 
c                     w.r.t. photon energy(k) & solid angle(\Omega_k) in
c                     cm^2 per atom per incident electron
c     signorm  Z^2 r_0^ /(8 \pi 137); units of cm^2

      REAL   sigma, k, mu, T0
      REAL   ksq, L, Delta0, p0, p0sq, eps, epsQ, Q, p, E, E0, T,
     ^     Qsq, beta, beta0, Cth0, Sth0, EE0, term(13), E0sq,
     ^     Esq, Delta0sq, Delta0_4, p0sqDsq, p0sqD4, Sth0sq
      REAL   signorm, Mevpermcsq
      COMMON / sigbrcom0 / signorm, Mevpermcsq
c
c     computation of much used factors
      ksq = k * k
      p0 = sqrt(T0 * (T0 + 2.))
      p0sq = p0 * p0
      E0 = 1. + T0
      E0sq = E0 * E0
      E = E0 - k
      Esq = E * E
      EE0 = E * E0
      T = E - 1.
      p =  sqrt(T * (T + 2.))
      beta0 = sqrt(1. - 1. / (E0 * E0))
      beta = sqrt(1. - 1. / (E * E))
      Cth0 = mu
      Qsq = p0sq + ksq - 2. * p0 * k * Cth0
      Q = sqrt(Qsq)
      Sth0 = sqrt(1. - mu * mu)
      Sth0sq = Sth0 * Sth0
      Delta0 = E0 - p0 * cth0
      Delta0sq = Delta0 * Delta0
      Delta0_4 = Delta0sq * Delta0sq
      p0sqDsq = p0sq * Delta0sq
      p0sqD4 = p0sq * Delta0_4
      eps = log((E + p) / (E - p))
      epsQ = log((Q + p) / (Q - p))
      L = log((EE0 - 1. + p * p0) / (EE0 - 1. - p * p0))
      term(1) = 8. * Sth0sq * ( 2. * E0sq + 1.) / p0sqD4
      term(2) = - 2. * (5. * E0sq + 2. * EE0 + 3. ) / p0sqDsq
      term(3) = - 2. * (p0sq - ksq) / (Qsq * Delta0sq)
      term(4) = 4. * E / (p0sq * Delta0)
      term(5) = 4. * E0 * Sth0sq * (3. * k - p0sq * E) / p0sqD4
      term(6) = 4. * E0sq * (E0sq + Esq) / p0sqDsq
      term(7) = 2. * (1. - (7. * E0sq - 3.* EE0 + Esq)) / p0sqDsq
      term(8) = 2. * k * (E0sq + EE0 - 1.) / (p0sq * Delta0)
      term(9) = - 4. * eps / (p * Delta0)
      term(10) = epsQ / (p * Q)
      term(11) = 4. / Delta0sq - 6. * k / Delta0 -
     ^      2. * k * (p0sq - ksq) / (Qsq * Delta0)
      term(12) = L / (p * p0)
      term(13) = signorm * p / (k * p0)
      sigma = term(13) * (
     ^     term(1) + term(2) + term(3) + term(4)
     ^     + term(12) * ( term(5) + term(6) + term(7) + term(8) )
     ^     + term(9) + term(10) * term(11)
     ^     ) 
      return
      END
c
c     graphs          --------------------------------------------------
c     End Xray Camera --------------------------------------------------
c
c
c     . ----------------------------------------------------------------
c     . ----------------------------------------------------------------
c     . Absorbers and Scintilator Treated Here -------------------------
c     . ----------------------------------------------------------------
c     . ----------------------------------------------------------------
c     .
c     Copyright D. W. Ignat, S. von Goeler, J. E. Stevens,
c     P. G. Roney, E. J. Valeo 
c     Princeton University, Plasma Physics Laboratory, 1992, 1993
c
c     ------------------------------------------------------------------
c
      SUBROUTINE GetXmnFac(nEs, EbyMeV, XmnFac, dcu, FC )
#include "Implic.inc"
#include "xparams.inc"
      INTEGER nEs
      REAL    EbyMeV(nEs), XmnFac(nEs)
      REAL DCU
      CHARACTER*2 FC
      CHARACTER*32 ErrorTxt
c                       ! DCU: thickness of foil (cm)
c                       ! FC:  FoilCode: CU,AG,TA,MO, or 00 for no foil
      INTEGER  nds, ndsDIM
      PARAMETER ( NDS = 12, ndsDIM = 12)
      REAL    dcm(nds)
      INTEGER i
      INTEGER iVACUUM,  iCARBON,iALUMINM,   iIRON, iNICKEL,
     ^                  iCOPPER, iIODINE, iCESIUM, iCSI,
     ^                  iTANTALUM, iMOLYBDENUM, iSILVER
      PARAMETER (iVACUUM = 1, iCARBON = 2, iALUMINM = 3)
      PARAMETER (iIRON = 4, iNICKEL = 5)
      PARAMETER (iCOPPER = 6, iIODINE = 7 , iCESIUM = 8, iCSI = 9)
      PARAMETER (iTANTALUM = 10, iMOLYBDENUM = 11, iSILVER = 12)
      INTEGER nSigInt
      PARAMETER (nSigInt = NENDIM)
      REAL    Ekv(nSigInt)
      REAL    rhoCsCSI, rhoICsI, rhoCarbon 
      DATA    rhoCsCSI, rhoICsI, rhoCarbon / 
     ^           2.307,   2.203,       1.0 /
c                                            !must be looked up
c     k Edges at  33.169,       35.985,  kV
c     These Data Give the Shield Always in Place in the Actual Experiment:
c     DATA    dcm(iVACUUM), dcm(iCARBON), dcm(iALUMINM) /
c    ^                0.0 ,         0.2 ,     0.37489   /
c     DATA    dcm(iIRON)  , dcm(iNICKEL), dcm(iCOPPER)  /
c    ^            0.0031  ,       0.0137,         0.0   /
c     DATA    dcm(iIODINE), dcm(iCESIUM), dcm(iCSI)     /
c    ^              0.0   ,         0.0 ,     0.0297    /
c     DATA  dcm(iTANTALUM),dcm(iMOLYBDENUM),dcm(iSILVER)/
c    ^              0.0   ,         0.0 ,         0.0   /
c
c     If we wanted to have phony foils to look at Photon Temperature
c     at low energy then SvG recomments the following:
c        10 kv 0.014 cm Al; 20 kv 0.109 cm Al; 
c        30 kv 0.335 cm Al; 40 kv 0.650 cm Al.
c

c * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

c     put in thicknesses of magnetic screen over the xray tube --
c     identical to data statements above
      dcm(iVACUUM)   = 0.0
      dcm(iCARBON)   = 0.2
      dcm(iALUMINM)  = 0.37489
      dcm(iIRON)     = 0.0031
      dcm(iNICKEL)   = 0.0137
      dcm(iMOLYBDENUM)= 0.0006 
c
c     put in active material on the screen
      dcm(iCSI)      = 0.0297
c
c     zero out thicknesses for optional materials -
      dcm(iCOPPER)    = 0.0
      dcm(iIODINE)    = 0.0
      dcm(iCESIUM)    = 0.0
      dcm(iTANTALUM)  = 0.0
      dcm(iSILVER)    = 0.0
c     .                                 determine subscript into dcm 
c     .                                 corresponding to foil code, and
c     .                                 put foil thickness into place;
c     .
c     .                                 If PhonyFoil, ie  FC='PF', then
c     .                                 zero out the default thickness for
c     .                                 everything, and put in an Aluminum foil
c     .                                 of the given thickness (SvGoeler Dec93)
      if (FC.eq.'CU') then
        dcm(iCOPPER) = dcu
      else if (FC.eq.'TA') then
        dcm(iTANTALUM) = dcu
      else if (FC.eq.'MO') then
        dcm(iMOLYBDENUM) = dcu + dcm(iMOLYBDENUM)
      else if (FC.eq.'AG') then
        dcm(iSILVER) = dcu
c     .                                Phony Foil Here
      else if (FC.eq.'PF') then
        do i=1,NDS
           dcm(i)=0.00
        enddo
c     .                                put back in thickness of the 
c     .                                aluminum phony foil 'PF'
           dcm(iALUMINM) = dcu
c     .                                put back in active screen material 
           dcm(iCSI)      = 0.0297
c     .
      else if (FC.eq.'00') then
        continue
      else
        write(ErrorTxt,
     ^      '(a2,'' !! Bad foil code in GetXmnFac'')') FC
        call LSCwarn (ErrorTxt)
      endif


c     .                                 Convert units of mc^2? or MeV to keV
      do 10 i = 1,nEs
        Ekv(i)  =  EbyMeV(i)*1000.
 10   continue

        call GetXmiss (nEs, Ekv, XmnFac,  nds, dcm )

c
cDBG        write(4,'(('' Ekv, Xmn: '', 2(1pe10.3,1x),/))')
cDBG     ^  (Ekv(i), XmnFac(i), i=1,nEs )
      return
      END
c
c     ------------------------------------------------------------------
c
      SUBROUTINE Absorb12(Energy, iNAME, 
     ^                   BarnPerA, MuByRho, nSigma, iError, ErrorTxt )
c     Reference:
c     X ray crosss sections based on W. H. McMaster, el al, UCRL 50174
c     Sec. II rev. 1
c     May 1969
c     \bibitem{mcmaster}
c     W. H. McMaster, \etal , UCRL 50174,
c     Sec. II rev. 1,  May 1969.

c     Copyright D. W. Ignat, S. von Goeler, J. E. Stevens,
c     P. G. Roney, E. J. Valeo 
c     Princeton University, Plasma Physics Laboratory, 1992, 1993
c     
#include "Implic.inc"
      INTEGER iNAME, iError
      CHARACTER*32  BLANK
      CHARACTER*(*) ErrorTxt
      REAL    Energy, BarnPerA, MuByRho, nSigma
      INTEGER iVACUUM,  iCARBON,iALUMINM,   iIRON, iNICKEL,
     ^                  iCOPPER, iIODINE, iCESIUM, iCSI,
     ^                  iTANTALUM, iMOLYBDENUM, iSILVER
      PARAMETER (iVACUUM = 1, iCARBON = 2, iALUMINM = 3)
      PARAMETER (iIRON = 4, iNICKEL = 5)
      PARAMETER (iCOPPER = 6, iIODINE = 7 , iCESIUM = 8, iCSI = 9)
      PARAMETER (iTANTALUM = 10, iMOLYBDENUM = 11, iSILVER = 12)
      INTEGER iLshell, iKshell,  iCoher ,  iInCoh
      INTEGER nLshells, nLatoms
      PARAMETER (iLshell = 1, iKshell = 2,  iCoher = 3, iInCoh = 4)
      PARAMETER (nLshells = 4, nLatoms = 12)
c     nLshells is a misnomer: 1 refers to L interactions below k edge
c     .                       2 refers to K interactions above k edge
c     .                       3 refers to coherent scattering
c     .                       4 refers to incoherent scattering
c     
c     If M shell interactions would be considered, then
c     nMshells would be 5: M, L, K, coherent, incoherent
c     Note, we are ignoring M shells in Cs and I
c
c
      REAL    kEdge(nLatoms), grPcc(nLatoms), AtmWt(nLatoms)
      REAL    Avogadro

      INTEGER iShell, i,j
      REAL    lnEnergy, lnSigma, epower(0:3)

      REAL    aL(4,nLshells,nLatoms)
      DATA    BLANK /'                                '/
c.    .              0123456789 123456789 123456789 12
      DATA    Avogadro / 6.02252e-01 /
c     DATA    Avogadro / 6.02252e-01 /  =  6.02 10^23 x 10^-24 for barns
      DATA     kEdge(iALUMINM), kEdge(iIRON), kEdge(iCOPPER)/
     ^                   1.560,        7.112,          8.979/
      DATA     kEdge(iIODINE ), kEdge(iCESIUM),kEdge(iNICKEL)/
     ^                  33.169,       35.985,          8.333 /
      DATA kEdge(iTANTALUM),kEdge(iMOLYBDENUM),kEdge(iSILVER)/
     ^                  67.414,       19.999,         25.514 /

      DATA     grPcc(iALUMINM), grPcc(iIRON), grPcc(iCOPPER)/
     ^                 2.702  ,       7.860 ,         8.940 /
      DATA     grPcc(iIODINE ), grPcc(iCESIUM),grPcc(iNICKEL)/  
     ^                 4.94   ,      1.873   ,        8.90   /
      DATA grPcc(iTANTALUM),grPcc(iMOLYBDENUM),grPcc(iSILVER) /
     ^                16.600,       10.220,          10.500 /

      DATA     AtmWt(iALUMINM), AtmWt(iIRON), AtmWt(iCOPPER)/
     ^                26.970  ,      55.850 ,        63.540 /
      DATA     AtmWt(iIODINE ), AtmWt(iCESIUM),AtmWt(iNICKEL)/
     ^                126.90  ,      132.910,        58.690  /
      DATA AtmWt(iTANTALUM),AtmWt(iMOLYBDENUM),AtmWt(iSILVER)/
     ^                180.950 ,       95.950,       107.880  /

      DATA     kEdge(iCARBON), grPcc(iCARBON) , AtmWt(iCARBON)  /
     ^                  0.00 ,         1.580  ,      12.010     /

      DATA   ((aL(i,j      ,iVACUUM),i = 1,4),j = 1,nLshells)  /
     ^        16*0.0                                       /

      DATA   ((aL(i,j,iCARBON ),i = 1,4),j = 1,nLshells)          /
     ^         0.0,        0.0,        0.0,        0.0,
     ^         1.06879e1, -2.71400,   -2.00530e-1, 2.07248e-2,
     ^         3.10861,   -2.60580e-1,-2.71974e-1, 1.35181e-2,
     ^        -9.82878e-1, 1.46693,   -2.93743e-1, 1.56005e-2 /

      DATA   ((aL(i,j,iALUMINM),i = 1,4),j = 1,nLshells)          /
     ^         1.08711E1, -2.77860,    1.75853e-1, 0.0,
     ^         1.31738e1, -2.18203,   -2.5896e-1,  2.2284e-2,
     ^         4.51995  ,  1.40549e-1,-3.5244e-1,  1.9369e-2,
     ^        -4.39322e-1, 1.30867 ,  -2.22648e-1, 7.54210e-3 /

      DATA   ((aL(i,j,iIRON   ),i = 1,4),j = 1,nLshells)          /
     ^         1.36696E1, -2.39195,   -1.37648E-1, 0.,
     ^         1.43456E1, -1.23491,   -4.18785E-1, 3.21662E-2,
     ^         5.93292,    2.25038E-1,-3.61748E-1, 1.93024E-2,
     ^        -3.42379E-1 ,1.57245,   -2.53198E-1, 9.85822E-3 /

      DATA   ((aL(i,j,iNICKEL ),i = 1,4),j = 1,nLshells)          /
     ^         1.39848e1, -2.48080,   -8.88115e-2, 0.0,
     ^         1.42388e1, -9.67736e-1,-4.78070e-1, 3.66138e-2,
     ^         6.09204  , 2.52277e-1, -3.66568e-1, 1.96586e-2,
     ^        -5.0436e-1, 1.70040,    -2.76443e-1, 1.12628e-2 /

      DATA   ((aL(i,j,iCOPPER ),i = 1,4),j = 1,nLshells)           /
     ^        14.2439,    -2.58677,   -6.67398E-2, 0.    ,
     ^        14.5808,    -1.18375,   -0.41385,    3.12088E-2 ,
     ^         6.17739,    2.73123E-1,-3.7236E-1,  2.01638E-2 ,
     ^        -5.7021E-1,  1.75042,   -2.84555E-1, 1.1693E-2  /

      DATA   ((aL(i,j,iIODINE ),i = 1,4),j = 1,nLshells)           /
     ^         1.64086e1, -2.48214,   -5.07179e-2, 0.0,
     ^         1.21075e1,  1.43635,   -8.82038e-1, 6.03575e-2,
     ^         7.27415,    3.77223e-1,-3.69728e-1, 1.86280e-2,
     ^        -4.04420e-2, 1.65596,   -2.510676,   9.04874e-3   /

      DATA   ((aL(i,j,iCESIUM ),i = 1,4),j = 1,nLshells)           /
     ^         1.65418e1, -2.46363,   -5.42849e-2, 0.0,
     ^         1.13757e1,  1.94161,   -9.83232e-1, 6.71986e-2,
     ^         7.33490,    3.76825e-1,-3.65713e-1, 1.81843e-2,
     ^         1.84861e-1, 1.50030,   -2.13333e-1, 6.24264e-3  /

      DATA  ((aL(i,j,iTANTALUM),i = 1,4),j = 1,nLshells)           /
     ^         1.72410E1, -2.30313,   -5.91006E-2, 0.0,
     ^         8.65271,    3.73117,   -1.26359,    8.23539E-2,
     ^         7.94534,    3.87299E-1,-3.47926E-1, 1.63299E-2,
     ^         1.96871E-1, 1.50623,   -1.91396E-1, 3.70889E-3 /

      DATA  ((aL(i,j,iSILVER),i = 1,4),j = 1,nLshells)             /
     ^         1.56869E+1,-2.22636,   -1.12223E-1, 0.0,
     ^         1.33926E+1, 4.41380E-1,-6.93711E-1, 4.82085E-2,
     ^         7.06446,    3.63456E-1,-3.73597E-1, 1.92478E-2,
     ^        -1.66475E-1, 1.65794,   -2.48740E-1, 8.66218E-3 /

      DATA  ((aL(i,j,iMOLYBDENUM),i = 1,4),j = 1,nLshells)         /
     ^         1.53494E+1,-2.26646,   -1.16881E-1, 0.0,
     ^         1.39853E+1,-1.17426E-1,-5.91094E-1, 4.17843E-2,
     ^         6.84600,    3.02797E-1,-3.51131E-1, 1.74403E-2,
     ^        -5.62860E-2, 1.55778,   -2.33341E-1, 7.85506E-3 /
c
c     ............................................. Execution begins here
c
      iError  =  0
      ErrorTxt= BLANK

      if (iNAME .eq. 1) then
c     .                                 The practice of including 'data' for
c     .                                 vacuum came from Jim Stevens, and we
c     .                                 are not sure why.  Here it is bypassed
        BarnPerA  =  0.
        MuByRho   =  0.
        nSigma    =  0.
        return
      endif

      if (Energy .lt. 1. ) then
        write(ErrorTxt,'('' invalid low energy: '',1pe10.3)')  Energy
        BarnPerA  =  0.
        MuByRho   =  0.
        nSigma    =  0.
        iError  =  1
        return
      endif

      if (Energy .gt. 1000. ) then
        write(ErrorTxt,'('' invalid high energy: '',1pe10.3)') Energy
        BarnPerA  =  0.
        MuByRho   =  0.
        nSigma    =  0.
        iError  =  2
        return
      endif

      if (iNAME .gt. 12) then
        write(ErrorTxt,'('' invalid atom: '', i4)')           iNAME
        BarnPerA  =  0.
        MuByRho   =  0.
        nSigma    =  0.
        iError  =  3
        return
      endif


c     !                                 Major branch based on number of shells
c     !                                 (branch not used; M shell ignored now)

      IF (iNAME .le. nLatoms) then
c     .                                 Use this code for L-shell atoms
        if (Energy .lt. kEdge(iNAME)) then
          iShell  =  1
        else
          iShell  =  2
        endif
c
        lnEnergy  =  alog(Energy)
        epower(0)  =  1.
        do 5 i = 1,3
          epower(i) = epower(i-1)*lnEnergy
 5      continue

          lnSigma  =  0.
        do 10 i = 1,4
          lnSigma  =  lnSigma + aL(i,iShell,iNAME)*epower(i-1)
  10    continue

        BarnPerA  =  exp(lnSigma)

        do 40 iShell = nLshells-1, nLshells
            lnSigma  =  0.
          do 30 i = 1,4
            lnSigma  =  lnSigma + aL(i,iShell,iNAME)*epower(i-1)
  30      continue
        BarnPerA  =  BarnPerA + exp(lnSigma)
  40    continue

c     .                                 Use this code for M-shell atoms
c     .                                 if and when this is added
      ELSE
        write(ErrorTxt,'('' invalid atom: '', i4)') iNAME
        BarnPerA  =  0.
        MuByRho   =  0.
        nSigma    =  0.
        iError  =  4
        return
      ENDIF
c     .                                 Finally, get N rho / AtWt * Sigma


      
      MuByRho   =  BarnPerA * Avogadro / AtmWt(iNAME)
      nSigma    =  MuByRho  * grPcc(iNAME)


      return
      END
c
c     ------------------------------------------------------------------
c
      SUBROUTINE GetXmiss (nSigInt, Eaxis, Xmiss,
     ^                     nds, dcm )
c     Copyright D. W. Ignat, S. von Goeler, J. E. Stevens,
c     P. G. Roney, E. J. Valeo 
c     Princeton University, Plasma Physics Laboratory, 1992, 1993
#include "Implic.inc"
      INTEGER nSigInt
      INTEGER nds
      CHARACTER*32 ErrorTxt
      REAL    dcm(nds)
      REAL    Eaxis(nSigInt), Xmiss(nSigInt)
      INTEGER iVACUUM,  iCARBON,iALUMINM,   iIRON, iNICKEL,
     ^                  iCOPPER, iIODINE, iCESIUM, iCSI,
     ^                  iTANTALUM, iMOLYBDENUM, iSILVER
      PARAMETER (iVACUUM = 1, iCARBON = 2, iALUMINM = 3)
      PARAMETER (iIRON = 4, iNICKEL = 5)
      PARAMETER (iCOPPER = 6, iIODINE = 7 , iCESIUM = 8, iCSI = 9)
      PARAMETER (iTANTALUM = 10, iMOLYBDENUM = 11, iSILVER = 12)

      INTEGER i, j, iError
      REAL    BarnPerA, MuByRho, nSigma
      REAL    MuRhoD, MuRhoCsI
      REAL    rhoCsCSI, rhoICsI, rhoCarbon 
      DATA    rhoCsCSI, rhoICsI, rhoCarbon / 
     ^           2.307,   2.203,       1.0 /
c                                            !must be looked up
c     k Edges at  33.169,       35.985,  kV
c
      do 70 i = 1,nSigInt
c
      MuRhoD    =  0.00
      MuRhoCsI  =  0.00

      if (Eaxis(i) .gt. 1000. ) then
        Xmiss(i) = 1.00
      else if(Eaxis(i) .lt. 1.00 ) then
        Xmiss(i) = 0.00
      else
c
        do 50 j = 1,nds
          if (dcm(j) .gt. 0. .and. j .ne. iCsI) then
            call Absorb12(Eaxis(i), j, 
     ^                   BarnPerA, MuByRho, nSigma, iError, ErrorTxt)
c
            if (j .eq. iCARBON) then
              MuRhoD  =  MuRhoD + MuByRho*rhoCarbon*dcm(j)
            else
              MuRhoD  =  MuRhoD + nSigma*dcm(j)
            endif
            if (iError .ge. 1) call LSCwarn(ErrorTxt)
          endif
c
          if (dcm(j) .gt. 0. .and. j .eq. iCSI) then
            call Absorb12(Eaxis(i), iIODINE, 
     ^                   BarnPerA, MuByRho, nSigma, iError, ErrorTxt)
c
            MuRhoCsI  =  MuRhoCsI + MuByRho*rhoICsI*dcm(j)
c
            call Absorb12(Eaxis(i), iCESIUM, 
     ^                   BarnPerA, MuByRho, nSigma, iError, ErrorTxt)
c
            MuRhoCsI  =  MuRhoCsI + MuByRho*rhoCsCsI*dcm(j)
c
            if (iError .ge. 1) call LSCwarn(ErrorTxt)
          endif
c
 50     continue
c
c
        if (MuRhoCsI .eq. 0.) then
          call LSCwarn(' no phosphor screen found; 1e-5 assumed')
          MuRhoCsI  =  1.e-05
        endif

        Xmiss(i)  =  exp ( - MuRhoD )*(1. - exp(-MuRhoCsI) )
      endif
 70   continue
c
      return 
      END

c     ------------------------------------------------------------------
c     . ----------------------------------------------------------------
c     . Absorbers and Scintilators End Here ----------------------------
c     . ----------------------------------------------------------------



c                                                                      |
c     -----------------------------------------------------------------|
c                                                                      |
      SUBROUTINE XRopen(GivnUnit, FileFrstNa)
#include "Implic.inc"
      INTEGER GivnUnit, SomeUnit, linepointer
      CHARACTER*1 PERIOD, SPACE
      CHARACTER*8  FileFrstNa, FileScndNa
      CHARACTER*8  mmddyy, hhmmss
      CHARACTER*17 FileCompNa
      CHARACTER*75 SomeString

      COMMON / XRrwcom / linepointer,SomeUnit
      DATA        PERIOD, SPACE / '.' , ' ' /

      CHARACTER*8  zzdate
      CHARACTER*10 zztime
      CHARACTER*5  zzzone
c
c     zzdate  ccyymmdd    cc=century yy=year mm=month dd=day
c     zztime  hhmmss.ttt  hh=hour mm=minute ss=second ttt=millisecond
c     zzzone  Shhmm       S=sign (-=west, +=east) hh=hour mm=minute
c
      INTEGER ivals(8)
c                             EXAMPLE
c     ivals ( 1 ) year        "2000"   
c     ivals ( 2 ) month       "10"
c     ivals ( 3 ) day         "12"
c     ivals ( 4 ) t - UTC     "-0400"
c     ivals ( 5 ) hour        "8"
c     ivals ( 6 ) minute      "21"
c     ivals ( 7 ) second      "23"
c     ivals ( 8 ) millisecond "621"
c
      call date_and_time(zzdate,zztime,zzzone,ivals)
      write(mmddyy,'(a)') zzdate(5:8)//zzdate(3:4)
      write(hhmmss,'(a)') zztime(1:6)

      SomeUnit = GivnUnit

      if (FileFrstNa .eq. 'XRinput') then
        open (SomeUnit,file='XRinput',
     ^        status='unknown')
        linepointer=0
      else
        FileScndNa = hhmmss
        open (SomeUnit,file=FileFrstNa//PERIOD//FileScndNa,
     ^        status='unknown',err=1300 )
        linepointer=0
        SomeString=FileFrstNa//SPACE//mmddyy//SPACE//hhmmss
        write(SomeUnit,'(a75)') SomeString
        linepointer=1
      endif
 1300 continue
c1300 continue need something real here
      return

      ENTRY      XRclos(GivnUnit)
      close(GivnUnit)
      return
      END

      SUBROUTINE XRdscw(xarray, yarray, npoints, SomeString)
#include "Implic.inc"
      INTEGER SomeUnit, linepointer
      INTEGER i,npoints
      REAL    xarray(npoints), yarray(npoints)
      CHARACTER*(*) SomeString
      COMMON / XRrwcom / linepointer,SomeUnit
      write(SomeUnit,'(a75)') SomeString
        linepointer=linepointer+1
      do 10 i=1,npoints
        write(SomeUnit,'(1x,1pe10.3,1x,1pe10.3)')xarray(i),yarray(i)
        linepointer=linepointer+1
 10   continue
      return
      END
c
c      SUBROUTINE XRdscr(xarray, yarray, npoints, SomeString)
c      IMPLICIT NONE
c      INTEGER SomeUnit, linepointer
c      INTEGER i,npoints
c      REAL    xarray(npoints), yarray(npoints)
c      CHARACTER*75 SomeString
c      COMMON / XRrwcom / linepointer,SomeUnit
c
c      if(linepointer .eq. 0) then
c        read(SomeUnit,'(a75)') SomeString
c        linepointer=linepointer+1
c      endif
c
c      read(SomeUnit,'(a75)') SomeString
c        linepointer=linepointer+1
c      do 10 i=1,npoints
c        read (SomeUnit,'(1x,1pe10.3,1x,1pe10.3)')xarray(i),yarray(i)
c        linepointer=linepointer+1
c 10   continue
c
c      return
c      END
#endif
